@@ -21,10 +21,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,USA
 
 ### global OpenSIPS commit stats, self-generated on each "rebuild-proj-stats"
-__PROJ_COMMITS=17783
-__PROJ_LINES_ADD=2184345
-__PROJ_LINES_DEL=1089621
-__LAST_REBUILD_SHA=b6ef99633e17d0fac08b98364389678f8ae3a3d2
+__PROJ_COMMITS=21604
+__PROJ_LINES_ADD=2576926
+__PROJ_LINES_DEL=1328694
+__LAST_REBUILD_SHA=816b58e6b36d873f63b20923fb822593a175b5f2
 
 TMP_FILE=/var/tmp/.opensips-build-contrib.tmp
 
@@ -242,7 +242,7 @@ fix_authors=(
   ["13e9a5cbe14050e622a3ef65cd34b72260a74f01"]="Kennard White"
   ["26599d25cbc140373a5c24759dce688235e57589"]="Anatoly Pidruchny"
 
-  # avpops
+  # sqlops
   ["37eba4b6d38f379a227040397c569f0d0fe99c9c"]="Kennard White"
   ["d129377f64f13e85ea0baf6d215092b4b4776f6e"]="Norman Brandinger"
   ["b9247c08af07662c6e712179dc57bcc5f16794aa"]="Kobi Eshun"
@@ -617,6 +617,8 @@ mod_renames=(
   [event_stream]=event_jsonrpc
   [b2b_logic]=b2b_logic:1605638778
   [b2b_logic_xml]=b2b_logic::1605638778
+  [sqlops]=dbops
+  [dbops]=avpops
 )
 
 mk_git_handle() {
@@ -652,14 +654,17 @@ normalize_arrays() {
   fi
 }
 
+# $1 (optional) - git SHA to be taken as a starting point
 rebuild_proj_commit_stats() {
   __PROJ_COMMITS=0
   __PROJ_LINES_ADD=0
   __PROJ_LINES_DEL=0
 
+  [ -n "$1" ] && commit_range="$1..HEAD"
+
   echo "Summing up all OpenSIPS commits! :-O"
 
-  for sha in $(git log --reverse --format=%H); do
+  for sha in $(git log --reverse --format=%H $commit_range); do
     [ -n "${skip_commits[$sha]}" ] && continue
 
     lines=($(git show $sha --format= --numstat \
@@ -672,6 +677,14 @@ rebuild_proj_commit_stats() {
     echo -en "\rProcessing commit #$__PROJ_COMMITS"
   done
 
+  if [ -n "$1" ]; then
+    echo "Commits: $__PROJ_COMMITS"
+    echo "Lines++: $__PROJ_LINES_ADD"
+    echo "Lines--: $__PROJ_LINES_DEL"
+    echo " ... since: $1"
+    return
+  fi
+
   sed -i "s/^__PROJ_COMMITS.*/__PROJ_COMMITS=$__PROJ_COMMITS/" $0
   sed -i "s/^__PROJ_LINES_ADD=.*/__PROJ_LINES_ADD=$__PROJ_LINES_ADD/" $0
   sed -i "s/^__PROJ_LINES_DEL=.*/__PROJ_LINES_DEL=$__PROJ_LINES_DEL/" $0
@@ -1001,7 +1014,7 @@ if [[ ! $(git log --reverse --format=%H | head -1) =~ ^f06ade ]]; then
 fi
 
 if [[ "$1" =~ rebuild-proj-stats ]]; then
-  rebuild_proj_commit_stats
+  rebuild_proj_commit_stats "$2"
   exit 0
 fi
 
