@@ -561,7 +561,7 @@ int parse_attr_def(char *line, FILE *fp)
 	unsigned int vendor_id = -1;
 	size_t buflen = strlen(line);
 	int i, len = buflen, attr_len = strlen("ATTRIBUTE"), name_len, avp_code;
-	char *name, *nt_name, *newp, *p = line, *end = p + len;
+	char *name, *nt_name = NULL, *newp, *p = line, *end = p + len;
 	enum dict_avp_basetype avp_type;
 	enum dict_avp_enc_type enc_type = AVP_ENC_TYPE_NONE;
 
@@ -581,6 +581,10 @@ int parse_attr_def(char *line, FILE *fp)
 		goto error;
 
 	nt_name = malloc(name_len + 1);
+	if (!nt_name) {
+		LOG_ERROR("Malloc failed\n");
+		return -1;
+	}
 	memcpy(nt_name, name, name_len);
 	nt_name[name_len] = '\0';
 
@@ -607,6 +611,9 @@ int parse_attr_def(char *line, FILE *fp)
 		        || (len >= strlen("hexstring") && !strncasecmp(p, STR_L("hexstring")))) {
 			avp_type = AVP_TYPE_OCTETSTRING;
 			enc_type = AVP_ENC_TYPE_HEX;
+		} else if (len >= strlen("time") && !strncasecmp(p, STR_L("time"))) {
+			avp_type = AVP_TYPE_OCTETSTRING;
+			enc_type = AVP_ENC_TYPE_TIME;
 		} else if ((len >= strlen("utf8string") && !strncasecmp(p, STR_L("utf8string")))
 		        || (len >= strlen("string") && !strncasecmp(p, STR_L("string"))))
 			avp_type = AVP_TYPE_OCTETSTRING;
@@ -659,11 +666,13 @@ int parse_attr_def(char *line, FILE *fp)
 
 		if (avp_count >= 128) {
 			LOG_ERROR("max AVP count exceeded (128)\n");
+			free(nt_name);
 			return -1;
 		}
 
 		if (parse_avp_def(avps, &avp_count, p, len) != 0) {
 			LOG_ERROR("failed to parse Grouped sub-AVP line: '%s'\n", line);
+			free(nt_name);
 			return -1;
 		}
 	}
@@ -674,6 +683,8 @@ create_avp:;
 	if (enc_type != AVP_ENC_TYPE_NONE &&
 			dm_enc_add((vendor_id != -1?vendor_id:0), avp_code, enc_type) != 0) {
 		LOG_ERROR("failed to add encoding type\n");
+		if (nt_name)
+			free(nt_name);
 		return -1;
 	}
 
@@ -723,6 +734,7 @@ create_avp:;
 	return 0;
 error:
 	LOG_ERROR("failed to parse line: %s\n", line);
+	free(nt_name);
 	return -1;
 }
 
