@@ -294,9 +294,9 @@ static int select_entire_dialog_table(db_res_t ** res, int *no_rows)
 
 
 
-struct socket_info * create_socket_info(db_val_t * vals, int n){
+const struct socket_info * create_socket_info(db_val_t * vals, int n){
 
-	struct socket_info * sock;
+	const struct socket_info * sock;
 	str p;
 
 	/* socket name */
@@ -420,6 +420,8 @@ void read_dialog_vars(char *b, int l, struct dlg_cell *dlg)
 
 	end = b + l;
 	p = b;
+
+	lock_start_write(dlg->vals_lock);
 	do {
 		/* read a new pair from input string */
 		p = read_pair( p, end, &name, &isval, &type);
@@ -432,6 +434,7 @@ void read_dialog_vars(char *b, int l, struct dlg_cell *dlg)
 			LM_ERR("failed to add val, skipping...\n");
 	} while(p!=end);
 
+	lock_stop_write(dlg->vals_lock);
 }
 
 
@@ -568,7 +571,7 @@ static int load_dialog_info_from_db(int dlg_hash_size)
 	str callid, from_uri, to_uri, from_tag, to_tag;
 	str cseq1,cseq2,contact1,contact2,rroute1,rroute2,mangled_fu,mangled_tu;
 	int no_rows = 10;
-	struct socket_info *caller_sock,*callee_sock;
+	const struct socket_info *caller_sock,*callee_sock;
 	int found_ended_dlgs=0;
 	unsigned int hash_entry,hash_id;
 	int_str tag_name;
@@ -1277,7 +1280,7 @@ static inline unsigned int write_pair( char *b, str *name, str *name_suffix,
 }
 
 
-str* write_dialog_vars( struct dlg_val *vars)
+str* write_dialog_vars( struct dlg_cell *dlg)
 {
 	static str o = {NULL,0};
 	static int o_l=0;
@@ -1286,8 +1289,10 @@ str* write_dialog_vars( struct dlg_val *vars)
 	char *p;
 	int intlen;
 
+	lock_start_read(dlg->vals_lock);
+
 	/* compute the required len */
-	for ( v=vars,l=0 ; v ; v=v->next) {
+	for ( v=dlg->vals,l=0 ; v ; v=v->next) {
 		l += v->name.len + 1/*'#'*/ + 1/*type char*/ + 1;/*'#'*/
 		if (v->type == DLG_VAL_TYPE_STR) {
 			l += v->val.s.len;
@@ -1311,16 +1316,19 @@ str* write_dialog_vars( struct dlg_val *vars)
 		if (o.s) pkg_free(o.s);
 		o.s = (char*)pkg_malloc(l);
 		if (o.s==NULL) {
+			lock_stop_read(dlg->vals_lock);
 			LM_ERR("not enough pkg mem (req=%d)\n",l);
 			return NULL;
 		}
 		o_l = l;
 	}
 
+	lock_stop_read(dlg->vals_lock);
+
 	/* write the stuff into it */
 	o.len = l;
 	p = o.s;
-	for ( v=vars ; v ; v=v->next) {
+	for ( v=dlg->vals ; v ; v=v->next) {
 		p += write_pair( p, &v->name,NULL, &v->val, v->type);
 	}
 	if (o.len!=p-o.s) {
@@ -1407,12 +1415,14 @@ int persist_reinvite_pinging(struct dlg_cell *dlg)
 		return 0;
 	}
 
+	lock_start_write(dlg->vals_lock);
+
 	val.s = dlg->legs[DLG_CALLER_LEG].in_sdp;
 	if (dlg->legs[DLG_CALLER_LEG].in_sdp.len &&
 			store_dlg_value_unsafe(dlg, &caller_in_sdp,
 				&val, DLG_VAL_TYPE_STR) != 0) {
 		LM_ERR("failed to persist caller UAC SDP\n");
-		return -1;
+		goto error;
 	}
 
 	val.s = dlg->legs[DLG_CALLER_LEG].out_sdp;
@@ -1420,14 +1430,14 @@ int persist_reinvite_pinging(struct dlg_cell *dlg)
 			store_dlg_value_unsafe(dlg, &caller_out_sdp,
 				&val, DLG_VAL_TYPE_STR) != 0) {
 		LM_ERR("failed to persist caller advertised SDP\n");
-		return -1;
+		goto error;
 	}
 
 	val.s = dlg->legs[DLG_CALLER_LEG].adv_contact;
 	if (store_dlg_value_unsafe(dlg, &caller_adv_ct,
 	                    &val, DLG_VAL_TYPE_STR) != 0) {
 		LM_ERR("failed to persist caller advertised Contact\n");
-		return -1;
+		goto error;
 	}
 
 	val.s = dlg->legs[dlg->legs_no[DLG_LEG_200OK]].in_sdp;
@@ -1435,7 +1445,7 @@ int persist_reinvite_pinging(struct dlg_cell *dlg)
 			store_dlg_value_unsafe(dlg, &callee_in_sdp,
 				&val, DLG_VAL_TYPE_STR) != 0) {
 		LM_ERR("failed to persist callee UAC SDP\n");
-		return -1;
+		goto error;
 	}
 
 	val.s = dlg->legs[dlg->legs_no[DLG_LEG_200OK]].out_sdp;
@@ -1443,17 +1453,22 @@ int persist_reinvite_pinging(struct dlg_cell *dlg)
 			store_dlg_value_unsafe(dlg, &callee_out_sdp,
 				&val, DLG_VAL_TYPE_STR) != 0) {
 		LM_ERR("failed to persist callee advertised SDP\n");
-		return -1;
+		goto error;
 	}
 
 	val.s = dlg->legs[dlg->legs_no[DLG_LEG_200OK]].adv_contact;
 	if (store_dlg_value_unsafe(dlg, &callee_adv_ct,
 	           &val, DLG_VAL_TYPE_STR) != 0) {
 		LM_ERR("failed to persist callee advertised Contact\n");
-		return -1;
+		goto error;
 	}
 
+	lock_stop_write(dlg->vals_lock);
 	return 0;
+
+error:
+	lock_stop_write(dlg->vals_lock);
+	return -1;
 }
 
 /* re-populate the SDPs/Contacts of caller/callee(s) from dlg val storage */
@@ -1574,7 +1589,7 @@ static inline void set_final_update_cols(db_val_t *vals, struct dlg_cell *cell,
 
 	/* save sharing tag name as dlg val */
 	val.s = cell->shtag;
-	if (cell->shtag.s && store_dlg_value_unsafe(cell, &shtag_dlg_val, &val,
+	if (cell->shtag.s && store_dlg_value(cell, &shtag_dlg_val, &val,
 		DLG_VAL_TYPE_STR) < 0)
 		LM_ERR("Failed to store sharing tag %.*s(%p) as dlg val\n",
 		       cell->shtag.len, cell->shtag.s, cell->shtag.s);
@@ -1583,7 +1598,7 @@ static inline void set_final_update_cols(db_val_t *vals, struct dlg_cell *cell,
 		if (cell->vals==NULL) {
 			VAL_NULL(vals) = 1;
 		} else {
-			s = write_dialog_vars( cell->vals );
+			s = write_dialog_vars( cell );
 			if (s==NULL) {
 				VAL_NULL(vals) = 1;
 			} else {
@@ -1859,7 +1874,7 @@ static int sync_dlg_db_mem(void)
 	int no_rows = 10;
 	unsigned int db_caller_cseq = 0, db_callee_cseq = 0;
 	unsigned int dlg_caller_cseq = 0, dlg_callee_cseq = 0;
-	struct socket_info *caller_sock,*callee_sock;
+	const struct socket_info *caller_sock,*callee_sock;
 	str callid, from_uri, to_uri, from_tag, to_tag;
 	str cseq1,cseq2,contact1,contact2,rroute1,rroute2,mangled_fu,mangled_tu;
 	unsigned int hash_entry, hash_id;
