@@ -235,10 +235,13 @@ static inline void free_dlg_dlg(struct dlg_cell *dlg)
 
 #ifdef DBG_DIALOG
 	sh_log(dlg->hist, DLG_DESTROY, "ref %d", dlg->ref);
-	sh_unref(dlg->hist);
-	dlg->hist = NULL;
+	if (dlg->hist) {
+		sh_unref(dlg->hist);
+		dlg->hist = NULL;
+	}
 #endif
 
+	lock_destroy_rw(dlg->vals_lock);
 	shm_free(dlg);
 }
 
@@ -321,17 +324,23 @@ struct dlg_cell* build_new_dlg( str *callid, str *from_uri, str *to_uri,
 
 	memset(dlg, 0, len);
 
+	dlg->vals_lock = lock_init_rw();
+	if (!dlg->vals_lock) {
+		LM_ERR("oom\n");
+		shm_free(dlg);
+		return NULL;
+	}
+
 #if defined(DBG_DIALOG)
 	dlg->hist = sh_push(dlg, dlg_hist);
 	if (!dlg->hist) {
 		LM_ERR("oom\n");
-		shm_free(dlg);
+		free_dlg_dlg(dlg);
 		return NULL;
 	}
 #endif
 
 	dlg->state = DLG_STATE_UNCONFIRMED;
-
 	dlg->h_entry = dlg_hash( callid);
 
 	LM_DBG("new dialog %p (c=%.*s,f=%.*s,t=%.*s,ft=%.*s) on hash %u\n",
@@ -384,7 +393,7 @@ int dlg_clone_callee_leg(struct dlg_cell *dlg, int cloned_leg_idx)
 }
 
 
-static inline int translate_contact_ipport( str *ct, struct socket_info *sock,
+static inline int translate_contact_ipport( str *ct, const struct socket_info *sock,
 																	str *dst)
 {
 	struct hdr_field ct_hdr;
@@ -392,7 +401,7 @@ static inline int translate_contact_ipport( str *ct, struct socket_info *sock,
 	contact_t *c;
 	struct sip_uri puri;
 	str hostport;
-	str *send_address_str, *send_port_str;
+	const str *send_address_str, *send_port_str;
 	char *p;
 
 	/* rely on the fact that the replicated hdr is well formated, so 
@@ -478,7 +487,7 @@ error:
    be no leg allocated, so automatically CALLER gets the first position, while
    the CALLEE legs will follow into the array in the same order they came */
 int dlg_update_leg_info(int leg_idx, struct dlg_cell *dlg, str* tag, str *rr,
-		str *contact, str *adv_ct, str *cseq, struct socket_info *sock,
+		str *contact, str *adv_ct, str *cseq, const struct socket_info *sock,
 		str *mangled_from,str *mangled_to,str *in_sdp, str *out_sdp)
 {
 	struct dlg_leg *leg;
@@ -879,7 +888,7 @@ struct dlg_cell* get_dlg_by_val(struct sip_msg *msg, str *attr, pv_spec_t *val)
 			LM_DBG("dlg in state %d to check\n",dlg->state);
 			if ( dlg->state>DLG_STATE_CONFIRMED )
 				continue;
-			if (check_dlg_value_unsafe(msg, dlg, attr, val)==0) {
+			if (check_dlg_value(msg, dlg, attr, val, 1)==0) {
 				ref_dlg_unsafe( dlg, 1);
 				dlg_unlock( d_table, d_entry);
 				return dlg;
@@ -1492,10 +1501,14 @@ static inline int internal_mi_print_dlg(mi_item_t *dialog_obj,
 		if (!context_obj)
 			goto error;
 
+		lock_start_read(dlg->vals_lock);
+
 		if (dlg->vals) {
 			values_arr = add_mi_array(context_obj, MI_SSTR("values"));
-			if (!values_arr)
+			if (!values_arr) {
+				lock_stop_read(dlg->vals_lock);
 				goto error;
+			}
 
 			/* print dlg values -> iterate the list */
 			for( dv=dlg->vals ; dv ; dv=dv->next) {
@@ -1528,10 +1541,16 @@ static inline int internal_mi_print_dlg(mi_item_t *dialog_obj,
 					}
 
 					values_item = add_mi_object(values_arr, NULL, 0);
-					if (!values_item)
+					if (!values_item) {
+						lock_stop_read(dlg->vals_lock);
 						goto error;
-					if (add_mi_string(values_item,dv->name.s,dv->name.len,p,j) < 0)
+					}
+
+					if (add_mi_string(values_item,dv->name.s,dv->name.len,p,j) < 0) {
+						lock_stop_read(dlg->vals_lock);
 						goto error;
+					}
+
 				} else {
 					values_item = add_mi_object(values_arr, NULL, 0);
 					if (!values_item)
@@ -1543,6 +1562,8 @@ static inline int internal_mi_print_dlg(mi_item_t *dialog_obj,
 			}
 		}
 
+		lock_stop_read(dlg->vals_lock);
+
 		/* print dlg profiles */
 		if (dlg->profile_links) {
 			profiles_arr = add_mi_array(context_obj, MI_SSTR("profiles"));
