@@ -28,25 +28,28 @@
 #include "../../dprint.h"
 #include "../../ut.h"
 #include "../../time_rec.h"
+#include "../../status_report.h"
 
 #include "dp_db.h"
 
-dp_head_p dp_hlist = NULL;
+dp_head_p dp_hlist;
+dp_head_p dp_df_head;
 
-str default_dp_db_url           =   {NULL, 0};
-str default_dp_table		=   {NULL, 0};
-str dp_table_name       	=   str_init(DP_TABLE_NAME);
-str dpid_column         	=   str_init(DPID_COL);
-str pr_column           	=   str_init(PR_COL);
-str match_op_column     	=   str_init(MATCH_OP_COL);
-str match_exp_column    	=   str_init(MATCH_EXP_COL);
-str match_flags_column  	=   str_init(MATCH_FLAGS_COL);
-str subst_exp_column    	=   str_init(SUBST_EXP_COL);
-str repl_exp_column     	=   str_init(REPL_EXP_COL);
-str disabled_column     	=   str_init(DISABLED_COL);
-str attrs_column        	=   str_init(ATTRS_COL);
-str timerec_column              =   str_init(TIMEREC_COL);
+str default_dp_db_url;
+str default_dp_table     =   str_init(DP_TABLE_NAME);
 
+str dpid_column          =   str_init(DPID_COL);
+str pr_column            =   str_init(PR_COL);
+str match_op_column      =   str_init(MATCH_OP_COL);
+str match_exp_column     =   str_init(MATCH_EXP_COL);
+str match_flags_column   =   str_init(MATCH_FLAGS_COL);
+str subst_exp_column     =   str_init(SUBST_EXP_COL);
+str repl_exp_column      =   str_init(REPL_EXP_COL);
+str disabled_column      =   str_init(DISABLED_COL);
+str attrs_column         =   str_init(ATTRS_COL);
+str timerec_column       =   str_init(TIMEREC_COL);
+
+extern void *dp_srg;
 
 #define GET_STR_VALUE(_res, _values, _index, _null)\
 	do{\
@@ -74,24 +77,24 @@ void list_rule(dpl_node_t * );
 void list_hash(dpl_id_t * , rw_lock_t *);
 
 
-dp_connection_list_p dp_conns = NULL;
+dp_connection_list_p dp_conns;
 
-int test_db(dp_connection_list_p dp_connection){
-
-	if (dp_connection->partition.s == 0) {
-		LM_ERR("invalid partition name\n");
+int test_db(dp_connection_list_p dp_connection)
+{
+	if (!dp_connection->partition.s) {
+		LM_ERR("NULL partition name\n");
 		return -1;
 	}
 
-	if (db_bind_mod(&dp_connection->db_url, &dp_connection->dp_dbf) < 0){
-		LM_ERR("unable to bind to a database driver\n");
+	if (db_bind_mod(&dp_connection->db_url, &dp_connection->dp_dbf) < 0) {
+		LM_ERR("failed to find a client driver for DB URL: '%s'\n",
+		       db_url_escape(&dp_connection->db_url));
 		return -1;
 	}
 
-	if (dp_connect_db(dp_connection) !=0)
+	if (dp_connect_db(dp_connection) != 0)
 		return -1;
 
-
 	if (db_check_table_version(&dp_connection->dp_dbf,
 		 *dp_connection->dp_db_handle, &dp_connection->table_name,
 			 DP_TABLE_VERSION) < 0) {
@@ -100,43 +103,9 @@ int test_db(dp_connection_list_p dp_connection){
 	}
 
 	dp_disconnect_db(dp_connection);
-
 	return 0;
 
 error:
-
-	dp_disconnect_db(dp_connection);
-	return -1;
-}
-
-
-int init_db_data(dp_connection_list_p dp_connection)
-{
-	if (dp_connection->partition.s == 0) {
-		LM_ERR("invalid partition name\n");
-		return -1;
-	}
-
-	if (dp_connect_db(dp_connection) !=0)
-		return -1;
-
-
-	if (db_check_table_version(&dp_connection->dp_dbf,
-		*dp_connection->dp_db_handle, &dp_connection->table_name,
-			DP_TABLE_VERSION) < 0) {
-		LM_ERR("error during table version check.\n");
-		goto error;
-	}
-
-
-	if(dp_load_db(dp_connection) != 0){
-		LM_ERR("failed to load database data\n");
-		goto error;
-	}
-
-	return 0;
-error:
-
 	dp_disconnect_db(dp_connection);
 	return -1;
 }
@@ -162,36 +131,74 @@ void dp_disconnect_db(dp_connection_list_p dp_conn)
 {
 	if (*dp_conn->dp_db_handle) {
 		dp_conn->dp_dbf.close(*dp_conn->dp_db_handle);
-		*dp_conn->dp_db_handle = 0;
+		*dp_conn->dp_db_handle = NULL;
 	}
 }
 
 
 int init_data(void)
 {
-	dp_head_p start, tmp = NULL;
+	dp_head_p start, tmp;
 
-	start = dp_hlist;
-	if (!start) {
-		LM_BUG("not even default partition defined!"
-				"An error occured!\n");
+	if (!dp_hlist) {
+		LM_ERR("no partition defined, not even the default one!\n");
 		return -1;
 	}
 
+	/* was the default partition re-pointed? */
+	if (!str_match(&dp_df_part, const_str(DEFAULT_PARTITION))) {
+		int found = 0;
+
+		for (start = dp_hlist; start; start = start->next) {
+			if (str_match(&dp_df_part, &start->partition)) {
+				found = 1;
+				break;
+			}
+		}
+
+		if (!found) {
+			LM_ERR("partition not found: '%.*s'\n",
+			       dp_df_part.len, dp_df_part.s);
+			return -1;
+		}
+	}
+
+	if (!dp_df_head) {
+		if (pkg_str_dup(&dp_df_part, &dp_hlist->partition) < 0) {
+			LM_ERR("oom\n");
+			return -1;
+		}
+
+		LM_INFO("no 'default' partition set, assuming '%.*s'\n",
+		        dp_df_part.len, dp_df_part.s);
+	}
+
+	start = dp_hlist;
 	while (start) {
 		LM_DBG("Adding partition with name [%.*s]\n",
 				start->partition.len, start->partition.s);
+
 		if (!dp_add_connection(start)) {
 			LM_ERR("failed to initialize partition '%.*s'\n",
 					start->partition.len, start->partition.s);
 			return -1;
 		}
 
+		if (sr_register_identifier( dp_srg, STR2CI(start->partition),
+				SR_STATUS_NO_DATA, CHAR_INT("no data loaded"), 20 ) ) {
+			LM_ERR("failed to create status report identifier for "
+				" partition \'%.*s\')\n",
+				start->partition.len, start->partition.s);
+			return -1;
+		}
+
 		tmp   = start;
 		start = start->next;
 		pkg_free(tmp);
 	}
 
+	dp_hlist = NULL;
+	dp_df_head = NULL;
 	return 0;
 }
 
@@ -206,16 +213,19 @@ void destroy_data(void)
 		destroy_hash(&el->hash[1]);
 		lock_destroy_rw(el->ref_lock);
 
+		shm_free(el->table_name.s);
+		shm_free(el->partition.s);
+		shm_free(el->db_url.s);
 		shm_free(el);
 	}
 }
 
-int dp_load_all_db(void)
+int dp_load_all_db(int initial)
 {
 	dp_connection_list_t *el;
 
 	for (el = dp_conns; el; el = el->next) {
-			if (dp_load_db(el) < 0) {
+			if (dp_load_db(el, initial) < 0) {
 					LM_ERR("unable to load %.*s table\n",
 							el->table_name.len, el->table_name.s);
 					return -1;
@@ -233,7 +243,7 @@ void dp_disconnect_all_db(void)
 }
 
 /*load rules from DB*/
-int dp_load_db(dp_connection_list_p dp_conn)
+int dp_load_db(dp_connection_list_p dp_conn, int initial)
 {
 	int i, nr_rows;
 	db_res_t * res = 0;
@@ -250,12 +260,14 @@ int dp_load_db(dp_connection_list_p dp_conn)
 
 	dpl_node_t *rule;
 	int no_rows = 10;
+	int loaded_rl=0, discarded_rl=0;
 
 
 	lock_start_write( dp_conn->ref_lock );
 
 	if( dp_conn->crt_index != dp_conn->next_index){
-		LM_WARN("a load command already generated, aborting reload...\n");
+		LM_WARN("a load command already in-progress for partition <%.*s>, "
+			"aborting reload...\n", dp_conn->partition.len, dp_conn->partition.s);
 		lock_stop_write( dp_conn->ref_lock );
 		return 0;
 	}
@@ -264,6 +276,15 @@ int dp_load_db(dp_connection_list_p dp_conn)
 
 	lock_stop_write( dp_conn->ref_lock );
 
+	sr_add_report( dp_srg, STR2CI(dp_conn->partition),
+		CHAR_INT("starting DB data loading"), 0);
+	if (initial)
+		sr_set_status( dp_srg,  STR2CI(dp_conn->partition),
+			SR_STATUS_LOADING_DATA, CHAR_INT("startup data loading"), 0);
+	else 
+		sr_set_status( dp_srg,  STR2CI(dp_conn->partition),
+			SR_STATUS_RELOADING_DATA, CHAR_INT("data re-loading"), 0);
+
 	if (dp_conn->dp_dbf.use_table(*dp_conn->dp_db_handle, &dp_conn->table_name) < 0){
 		LM_ERR("error in use_table\n");
 		goto err1;
@@ -308,7 +329,8 @@ int dp_load_db(dp_connection_list_p dp_conn)
 
 
 	if(nr_rows == 0){
-		LM_WARN("no data in the db\n");
+		LM_DBG("no data in the db for partition <%.*s>\n",
+			dp_conn->partition.len, dp_conn->partition.s);
 		goto end;
 	}
 
@@ -318,7 +340,10 @@ int dp_load_db(dp_connection_list_p dp_conn)
 			values = ROW_VALUES(rows+i);
 
 			if ((rule = build_rule(values)) == NULL) {
-				LM_WARN(" failed to build rule -> skipping\n");
+				LM_WARN(" failed to build rule |%s| in partition <%.*s>"
+					" -> skipping\n", VAL_STRING(values+3),
+					dp_conn->partition.len, dp_conn->partition.s);
+				discarded_rl++;
 				continue;
 			}
 
@@ -328,6 +353,7 @@ int dp_load_db(dp_connection_list_p dp_conn)
 				LM_ERR("add_rule2hash failed\n");
 				goto err2;
 			}
+			loaded_rl++;
 		}
 
 
@@ -360,9 +386,19 @@ end:
 	list_hash(dp_conn->hash[dp_conn->crt_index], dp_conn->ref_lock);
 
 	dp_conn->dp_dbf.free_result(*dp_conn->dp_db_handle, res);
+
+	/* do the reporting */
+	sr_add_report( dp_srg, STR2CI(dp_conn->partition),
+		CHAR_INT("DB data loading successfully completed"), 0);
+	sr_add_report_fmt( dp_srg, STR2CI(dp_conn->partition), 0,
+		"%d rules loaded (%d discarded)", loaded_rl, discarded_rl);
+	sr_set_status( dp_srg, STR2CI(dp_conn->partition), SR_STATUS_READY,
+		CHAR_INT("data available"), 0);
+
 	return 0;
 
 err1:
+	LM_ERR("DB reload failed on partition <%.*s>\n",dp_conn->partition.len, dp_conn->partition.s);
 
 	lock_start_write( dp_conn->ref_lock );
 
@@ -370,9 +406,20 @@ err1:
 
 	lock_stop_write( dp_conn->ref_lock );
 
+	sr_add_report( dp_srg, STR2CI(dp_conn->partition),
+		CHAR_INT("DB data loading failed, discarding"), 0);
+	if (initial)
+		sr_set_status( dp_srg, STR2CI(dp_conn->partition), SR_STATUS_NO_DATA,
+			CHAR_INT("no data loaded"), 0);
+	else
+		sr_set_status( dp_srg, STR2CI(dp_conn->partition), SR_STATUS_READY,
+			CHAR_INT("data available"), 0);
+
 	return -1;
 
 err2:
+	LM_ERR("DB reload failed on partition <%.*s>\n",dp_conn->partition.len, dp_conn->partition.s);
+
 	if(rule)	destroy_rule(rule);
 	destroy_hash(&dp_conn->hash[dp_conn->next_index]);
 	dp_conn->dp_dbf.free_result(*dp_conn->dp_db_handle, res);
@@ -383,6 +430,16 @@ err2:
 	/* if lock defined - release the exclusive writing access */
 
 	lock_stop_write( dp_conn->ref_lock );
+
+	sr_add_report( dp_srg, STR2CI(dp_conn->partition),
+		CHAR_INT("DB data loading failed, discarding"), 0);
+	if (initial)
+		sr_set_status( dp_srg, STR2CI(dp_conn->partition), SR_STATUS_READY,
+			CHAR_INT("data available"), 0);
+	else
+		sr_set_status( dp_srg, STR2CI(dp_conn->partition), SR_STATUS_NO_DATA,
+			CHAR_INT("no data loaded"), 0);
+
 	return -1;
 }
 
@@ -405,51 +462,11 @@ int str_to_shm(str src, str * dest)
 	return 0;
 }
 
-static inline tmrec_t* parse_time_def(char *time_str) {
-
-	tmrec_p time_rec;
-	char *p,*s;
-
-	p = time_str;
-	time_rec = 0;
-
-	time_rec = tmrec_new(SHM_ALLOC);
-	if (time_rec==0) {
-		LM_ERR("no more shm mem\n");
-		goto error;
-	}
-
-	/* empty definition? */
-	if ( time_str==0 || *time_str==0 )
-		goto done;
-
-	load_TR_value( p, s, time_rec, tr_parse_dtstart, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_duration, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_freq, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_until, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_interval, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_byday, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_bymday, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_byyday, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_byweekno, parse_error, done);
-	load_TR_value( p, s, time_rec, tr_parse_bymonth, parse_error, done);
-
-	/* success */
-done:
-	return time_rec;
-parse_error:
-	LM_ERR("parse error in <%s> around position %i\n",
-		time_str, (int)(long)(p-time_str));
-error:
-	if (time_rec)
-		tmrec_free( time_rec );
-	return 0;
-}
 
 /*compile the expressions, and if ok, build the rule */
 dpl_node_t * build_rule(db_val_t * values)
 {
-	tmrec_t *parsed_timerec;
+	tmrec_expr *parsed_timerec;
 	pcre * match_comp, *subst_comp;
 	struct subst_expr * repl_comp;
 	dpl_node_t * new_rule;
@@ -557,7 +574,7 @@ dpl_node_t * build_rule(db_val_t * values)
 	/* Retrieve and Parse Timerec Matching Pattern */
 	GET_STR_VALUE(timerec, values, 8, 1);
 	if( !VAL_NULL(values+8) && timerec.len && timerec.s) {
-		parsed_timerec = parse_time_def(timerec.s);
+		parsed_timerec = tmrec_expr_parse(timerec.s, SHM_ALLOC);
 		if(!parsed_timerec) {
 			LM_ERR("failed to parse timerec pattern %.*s\n",
 				timerec.len, timerec.s);
@@ -740,7 +757,7 @@ void destroy_rule(dpl_node_t * rule){
 		shm_free(rule->timerec.s);
 
 	if(rule->parsed_timerec)
-		shm_free(rule->parsed_timerec);
+		tmrec_expr_free(rule->parsed_timerec);
 }
 
 
@@ -805,14 +822,13 @@ void list_rule(dpl_node_t * rule)
 }
 
 /* Retrieves the corresponding entry of the given partition name */
-dp_connection_list_p dp_get_connection(str * partition)
+dp_connection_list_p dp_get_connection(str *partition)
 {
 	dp_connection_list_t *el;
 
 	el = dp_conns;
-	while (el && str_strcmp(partition, &el->partition)) {
+	while (el && str_strcmp(partition, &el->partition))
 		el = el->next;
-	}
 
 	return el;
 }
@@ -847,14 +863,12 @@ dp_connection_list_p dp_add_connection(dp_head_p head)
 		return NULL;
 	}
 
-	/*Set table name*/
-	el->table_name = head->dp_table_name;
-
-	/*Set partition*/
-	el->partition = head->partition;
-
-	/*Set db_url*/
-	el->db_url = head->dp_db_url;
+	if (shm_str_dup(&el->table_name, &head->dp_table_name) != 0 ||
+	        shm_str_dup(&el->partition, &head->partition) != 0 ||
+	        shm_str_dup(&el->db_url, &head->dp_db_url) != 0) {
+		LM_ERR("oom\n");
+		return NULL;
+	}
 
 	el->dp_db_handle = pkg_malloc(sizeof(db_con_t*));
 	if (!el->dp_db_handle) {
@@ -878,6 +892,5 @@ dp_connection_list_p dp_add_connection(dp_head_p head)
 	LM_DBG("Added dialplan partition [%.*s] table [%.*s].\n",
 		 head->partition.len, head->partition.s,
 				head->dp_table_name.len, head->dp_table_name.s);
-
 	return el;
 }
