@@ -57,7 +57,7 @@
 
 extern ds_partition_t *partitions;
 
-extern struct socket_info *probing_sock;
+extern const struct socket_info *probing_sock;
 extern event_id_t dispatch_evi_id;
 extern ds_partition_t *default_partition;
 
@@ -174,7 +174,7 @@ void ds_destroy_data(ds_partition_t *partition)
 }
 
 
-int add_dest2list(int id, str uri, struct socket_info *sock, str *comsock, int state,
+int add_dest2list(int id, str uri, const struct socket_info *sock, str *comsock, int state,
 			int weight, int prio, int probe_mode, str attrs, str description, ds_data_t *d_data)
 {
 	ds_dest_p dp = NULL;
@@ -839,6 +839,12 @@ int ds_route_algo(struct sip_msg *msg, ds_set_p set,
 		fret = run_route_algo(msg, algo_route->idx, &set->dlist[i]);
 		set->dlist[i].route_algo_value = fret;
 
+		if (fret < 0) {
+			/* move to the end of the list */
+			sset[end_idx--] = &set->dlist[i];
+			continue;
+		}
+
 		/* search the proper position */
 		j = 0;
 		for (; j < cnt && sset[j]->route_algo_value <= fret; j++);
@@ -968,7 +974,7 @@ void ds_flusher_routine(unsigned int ticks, void* param)
 
 	ds_partition_t *partition;
 	for (partition = partitions; partition; partition = partition->next){
-		if (*partition->db_handle==NULL)
+		if (*partition->db_handle==NULL || !partition->persistent_state)
 			continue;
 
 		val_cmp[0].type = DB_INT;
@@ -1033,7 +1039,7 @@ void ds_flusher_routine(unsigned int ticks, void* param)
 
 
 /*load groups of destinations from DB*/
-static ds_data_t* ds_load_data(ds_partition_t *partition, int use_state_col)
+static ds_data_t* ds_load_data(ds_partition_t *partition)
 {
 	ds_data_t *d_data;
 	int i, id, nr_rows, cnt, nr_cols = 9;
@@ -1041,7 +1047,7 @@ static ds_data_t* ds_load_data(ds_partition_t *partition, int use_state_col)
 	int weight;
 	int prio;
 	int probe_mode;
-	struct socket_info *sock;
+	const struct socket_info *sock;
 	str uri;
 	str attrs, weight_st;
 	str description;
@@ -1055,7 +1061,7 @@ static ds_data_t* ds_load_data(ds_partition_t *partition, int use_state_col)
 			&ds_dest_prio_col, &ds_dest_description_col,
 			&ds_dest_probe_mode_col, &ds_dest_state_col};
 
-	if (!use_state_col)
+	if (!partition->persistent_state)
 		nr_cols--;
 
 	if(*partition->db_handle == NULL){
@@ -1157,7 +1163,7 @@ static ds_data_t* ds_load_data(ds_partition_t *partition, int use_state_col)
 			probe_mode = VAL_INT(values+7);
 
 		/* state */
-		if (!use_state_col || VAL_NULL(values+8))
+		if (!partition->persistent_state || VAL_NULL(values+8))
 			/* active state */
 			state = 0;
 		else
@@ -1220,7 +1226,7 @@ int ds_reload_db(ds_partition_t *partition, int initial, int is_inherit_state)
 		sr_set_status( ds_srg, STR2CI(partition->name),
 			SR_STATUS_RELOADING_DATA, CHAR_INT("data re-loading"), 0);
 
-	new_data = ds_load_data(partition, ds_persistent_state);
+	new_data = ds_load_data(partition);
 	if (new_data==NULL) {
 		LM_ERR("failed to load the new data, dropping the reload\n");
 		if (initial)
@@ -1628,7 +1634,7 @@ static inline int ds_get_index(int group, ds_set_p *index,
 }
 
 
-int ds_update_dst(struct sip_msg *msg, str *uri, struct socket_info *sock,
+int ds_update_dst(struct sip_msg *msg, str *uri, const struct socket_info *sock,
 																	int mode)
 {
 	uri_type utype;
@@ -2130,7 +2136,7 @@ error:
 
 int ds_next_dst(struct sip_msg *msg, int mode, ds_partition_t *partition)
 {
-	struct socket_info *sock;
+	const struct socket_info *sock;
 	struct usr_avp *avp;
 	struct usr_avp *tmp_avp;
 	struct usr_avp *attr_avp;
@@ -2693,7 +2699,7 @@ void ds_check_timer(unsigned int ticks, void* param)
 		 * to free the whole structure here */
 		ds_options_callback_param_t params;
 
-		struct socket_info *sock;
+		const struct socket_info *sock;
 		struct usr_avp *avps;
 
 		struct gw_prob_pack *next;
@@ -2802,7 +2808,9 @@ void ds_check_timer(unsigned int ticks, void* param)
 
 			/* Execute the Dialog using the "request"-Method of the
 			 * TM-Module.*/
-			if (tmb.new_auto_dlg_uac(&ds_ping_from,
+			if (tmb.new_auto_dlg_uac((partition->ping_from.len?
+							&partition->ping_from:
+							&ds_ping_from),
 			&pack->params.uri, NULL, NULL,
 			pack->sock?pack->sock:probing_sock,
 			&dlg) != 0 ) {
@@ -2818,14 +2826,23 @@ void ds_check_timer(unsigned int ticks, void* param)
 			dlg->avps = pack->avps;
 			pack->avps = NULL;
 
-			if (tmb.t_request_within(&ds_ping_method,
+			if (tmb.t_request_within((partition->ping_method.len?
+							&partition->ping_method:
+							&ds_ping_method),
 					NULL,
 					NULL,
 					dlg,
 					ds_options_callback,
 					(void*)pack,
-					osips_shm_free) < 0) {
-				LM_ERR("unable to execute dialog\n");
+					osips_shm_free) < 0)
+			{
+				LM_ERR("failed to send probe for <%.*s>, set %d, setting "
+					"it to probing\n",
+					pack->params.uri.len, pack->params.uri.s,
+					pack->params.set_id);
+				ds_set_state( pack->params.set_id, &pack->params.uri,
+					DS_PROBING_DST, 1, pack->params.partition, 1, 0,
+					MI_SSTR("failed to send probe"));
 				shm_free(pack);
 			}
 			tmb.free_dlg(dlg);
