@@ -60,8 +60,6 @@
 
 #define MI_FULL_LISTING (1<<0)
 
-extern int ds_persistent_state;
-
 typedef struct _ds_dest
 {
 	str uri;        /* URI used in pinging and for matching destination at reload */
@@ -75,7 +73,7 @@ typedef struct _ds_dest
 	unsigned short running_weight;
 	unsigned short active_running_weight;
 	unsigned short priority;
-	struct socket_info *sock;
+	const struct socket_info *sock;
 	struct ip_addr ips[DS_MAX_IPS]; /* IP-Address of the entry */
 	unsigned short int ports[DS_MAX_IPS]; /* Port of the request URI */
 	unsigned short int protos[DS_MAX_IPS]; /* Protocol of the request URI */
@@ -119,6 +117,9 @@ typedef struct _ds_partition
 	str name;              /* Partition name */
 	str table_name;        /* Table name */
 	str db_url;            /* DB url */
+	str ping_from;
+	str ping_method;
+	int persistent_state;
 
 	db_con_t **db_handle;
 	db_func_t dbf;
@@ -172,7 +173,7 @@ typedef struct
 typedef struct _ds_selected_dst
 {
 	str uri;
-	struct socket_info *socket;
+	const struct socket_info *socket;
 } ds_selected_dst, *ds_selected_dst_p;
 
 extern str ds_set_id_col;
@@ -217,7 +218,7 @@ int ds_reload_db(ds_partition_t *partition, int initial, int is_inherit_state);
 int init_ds_data(ds_partition_t *partition);
 void ds_destroy_data(ds_partition_t *partition);
 
-int ds_update_dst(struct sip_msg *msg, str *uri, struct socket_info *sock, int mode);
+int ds_update_dst(struct sip_msg *msg, str *uri, const struct socket_info *sock, int mode);
 int ds_select_dst(struct sip_msg *msg, ds_select_ctl_p ds_select_ctl, ds_selected_dst_p selected_dst, int ds_flags);
 int ds_next_dst(struct sip_msg *msg, int mode, ds_partition_t *partition);
 int ds_set_state(int group, str *address, int state, int type,
