@@ -66,13 +66,13 @@ static int cclen(const char *number)
 		case '7':
 			return(1);
 		case '2':
-			if ((d2 == '0') || (d1 == '7'))
+			if ((d2 == '0') || (d2 == '7'))
 				return(2);
 			break;
 		case '3':
-			if ((d2 >= '0') && (d1 <= '4'))
+			if ((d2 >= '0') && (d2 <= '4'))
 				return(2);
-			if ((d2 == '6') || (d1 == '9'))
+			if ((d2 == '6') || (d2 == '9'))
 				return(2);
 			break;
 		case '4':
@@ -80,19 +80,19 @@ static int cclen(const char *number)
 				return(2);
 			break;
 		case '5':
-			if ((d2 >= '1') && (d1 <= '8'))
+			if (d2 != '9')
 				return(2);
 			break;
 		case '6':
-			if (d1 <= '6')
+			if (d2 <= '6')
 				return(2);
 			break;
 		case '8':
-			if ((d2 == '1') || (d1 == '2') || (d1 == '4') || (d1 == '6'))
+			if ((d2 == '1') || (d2 == '2') || (d2 == '4') || (d2 == '6'))
 				return(2);
 			break;
 		case '9':
-			if (d1 <= '5')
+			if (d2 <= '5')
 				return(2);
 			if (d2 == '8')
 				return(2);
@@ -221,47 +221,13 @@ static inline int sip_match( struct naptr_rdata* naptr, str* service)
 }
 
 
-/*
- * Checks if argument is an e164 number starting with +
- */
-static inline int is_e164(str* _user)
-{
-	int i;
-	char c;
 
-	if ((_user->len > 2) && (_user->len < 17) && ((_user->s)[0] == '+')) {
-		for (i = 1; i < _user->len; i++) {
-			c = (_user->s)[i];
-			if ((c < '0') || (c > '9')) return -1;
-		}
-		return 1;
-	} else {
-	    return -1;
-	}
-}
-
-
-/*
- * Call is_from_user_enum_2 with module parameter suffix and default service.
- */
-int is_from_user_enum_0(struct sip_msg* _msg, char* _str1, char* _str2)
-{
-	return is_from_user_enum_2(_msg, (char *)(&suffix), (char *)(&service));
-}
-
-/*
- * Call is_from_user_enum_2 with given suffix and default service.
- */
-int is_from_user_enum_1(struct sip_msg* _msg, char* _suffix, char* _str2)
-{
-	return is_from_user_enum_2(_msg, _suffix, (char *)(&service));
-}
 
 /*
  * Check if from user is a valid enum based user, and check to make sure
  * that the src_ip == an srv record that maps to the enum from user.
  */
-int is_from_user_enum_2(struct sip_msg* _msg, char* _suffix, char* _service)
+int is_from_user_enum(struct sip_msg* _msg, str* suffix, str* service)
 {
 	struct ip_addr addr;
 	struct hostent* he;
@@ -275,9 +241,6 @@ int is_from_user_enum_2(struct sip_msg* _msg, char* _suffix, char* _service)
 	struct sip_uri luri;
 	struct rdata* head;
 
-	str* suffix;
-	str* service;
-
 	struct rdata* l;
 	struct naptr_rdata* naptr;
 
@@ -299,9 +262,6 @@ int is_from_user_enum_2(struct sip_msg* _msg, char* _suffix, char* _service)
 	    return -1;
 	}
 
-	suffix = (str*)_suffix;
-	service = (str*)_service;
-
 	if (is_e164(&(furi->user)) == -1) {
 	    LM_ERR("From URI user is not an E164 number\n");
 	    return -1;
@@ -679,104 +639,32 @@ done:
 }
 
 
-/*
- * Call enum_query_2 with module parameter suffix and default service.
- */
-int enum_query_0(struct sip_msg* _msg, char* _str1, char* _str2)
-{
-	return enum_query_2(_msg, NULL, NULL);
-}
-
-
-/*
- * Call enum_query_2 with given suffix and default service.
- */
-int enum_query_1(struct sip_msg* _msg, char* _suffix, char* _str2)
-{
-	return enum_query_2(_msg, _suffix, NULL);
-}
-
-
 /*
  * See documentation in README file.
  */
-int enum_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
+int enum_query(struct sip_msg* _msg, str* _suffix, str* _service, str* _num)
 {
 	char *user_s;
 	int user_len, i, j;
 	char name[MAX_DOMAIN_SIZE];
 	char string[17];
-	gparam_p gp = NULL;
-	pv_value_t value;
 
-	str __suffix = {0, 0}, __service = {0, 0};
-
-	/* Use the suffix module parameter */
-	if (_suffix == NULL) {
-		__suffix.s = suffix.s;
-		__suffix.len = suffix.len;
-	} else {
-		gp = (gparam_p) _suffix;
-
-		if (gp->type == GPARAM_TYPE_PVS) {
-			if (pv_get_spec_value(_msg, gp->v.pvs, &value) != 0 ||
-				value.flags & PV_VAL_NULL || value.flags & PV_VAL_EMPTY) {
-				LM_ERR("No PV or NULL value specified for suffix\n");
-				return E_CFG;
-			}
-
-			if (value.flags & PV_VAL_STR) {
-				__suffix.s = value.rs.s;
-				__suffix.len = value.rs.len;
-			} else {
-				LM_ERR("Unsupported PV value type\n");
-				return E_CFG;
-			}
-		} else if (gp->type == GPARAM_TYPE_STR) {
-			__suffix.s = gp->v.sval.s;
-			__suffix.len = gp->v.sval.len;
+	if (!_num) {
+		if (parse_sip_msg_uri(_msg) < 0) {
+			LM_ERR("Parsing of R-URI failed\n");
+			return -1;
 		}
+
+		_num = &_msg->parsed_uri.user;
 	}
 
-	/* Use the internal service */
-	if (_service == NULL) {
-		__service.s = service.s;
-		__service.len = service.len;
-	} else {
-		gp = (gparam_p) _service;
-
-		if (gp->type == GPARAM_TYPE_PVS) {
-			if (pv_get_spec_value(_msg, gp->v.pvs, &value) != 0 ||
-				value.flags & PV_VAL_NULL || value.flags & PV_VAL_EMPTY) {
-				LM_ERR("No PV or NULL value specified for suffix\n");
-				return E_CFG;
-			}
-
-			if (value.flags & PV_VAL_STR) {
-				__service.s = value.rs.s;
-				__service.len = value.rs.len;
-			} else {
-				LM_ERR("Unsupported PV value type\n");
-				return E_CFG;
-			}
-		} else if (gp->type == GPARAM_TYPE_STR) {
-			__service.s = gp->v.sval.s;
-			__service.len = gp->v.sval.len;
-		}
-	}
-
-	if (parse_sip_msg_uri(_msg) < 0) {
-		LM_ERR("Parsing of R-URI failed\n");
+	if (is_e164(_num) == -1) {
+		LM_ERR("number is not in E164 format\n");
 		return -1;
 	}
 
-	if (is_e164(&(_msg->parsed_uri.user)) == -1) {
-		LM_ERR("R-URI user is not an E164 number\n");
-		return -1;
-	}
-
-	user_s = _msg->parsed_uri.user.s;
-	user_len = _msg->parsed_uri.user.len;
+	user_s = _num->s;
+	user_len = _num->len;
 
 	memcpy(&(string[0]), user_s, user_len);
 	string[user_len] = (char)0;
@@ -788,34 +676,16 @@ int enum_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
 		j = j + 2;
 	}
 
-	memcpy(name + j, __suffix.s, __suffix.len + 1);
+	memcpy(name + j, _suffix->s, _suffix->len + 1);
 
-	return do_query(_msg, string, name, &__service);
-}
-
-
-/*
- * Call isn_query_2 with module parameter suffix and default service.
- */
-int isn_query_0(struct sip_msg* _msg, char* _str1, char* _str2)
-{
-	return isn_query_2(_msg, (char *)(&isnsuffix), (char *)(&service));
-}
-
-
-/*
- * Call isn_query_2 with given suffix and default service.
- */
-int isn_query_1(struct sip_msg* _msg, char* _suffix, char* _str2)
-{
-	return isn_query_2(_msg, _suffix, (char *)(&service));
+	return do_query(_msg, string, name, _service);
 }
 
 
 /*
  * See documentation in README file.
  */
-int isn_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
+int isn_query(struct sip_msg* _msg, str* suffix, str* service)
 {
 	char *user_s = NULL;
 	int user_len, i, j;
@@ -824,11 +694,6 @@ int isn_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
 	char szItad[17] = {0};
 	size_t nItlen = 0;
 
-	str *suffix, *service;
-
-	suffix = (str*)_suffix;
-	service = (str*)_service;
-
 	if (parse_sip_msg_uri(_msg) < 0) {
 		LM_ERR("Parsing of R-URI failed\n");
 		return -1;
@@ -874,24 +739,7 @@ int isn_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
 
 /*********** INFRASTRUCTURE ENUM ***************/
 
-/*
- * Call enum_query_2 with default suffix and service.
- */
-int i_enum_query_0(struct sip_msg* _msg, char* _suffix, char* _service)
-{
-	return i_enum_query_2(_msg, (char *)(&i_suffix), (char *)(&service));
-}
-
-/*
- * Call enum_query_2 with given suffix and default service.
- */
-int i_enum_query_1(struct sip_msg* _msg, char* _suffix, char* _service)
-{
-	return i_enum_query_2(_msg, _suffix, (char *)(&service));
-}
-
-
-int i_enum_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
+int i_enum_query(struct sip_msg* _msg, str* suffix, str* service)
 {
 	char *user_s;
 	int user_len, i, j;
@@ -904,11 +752,6 @@ int i_enum_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
 
 	char string[17];
 
-	str *suffix, *service;
-
-	suffix = (str*)_suffix;
-	service = (str*)_service;
-
 	if (parse_sip_msg_uri(_msg) < 0) {
 		LM_ERR("Parsing of R-URI failed\n");
 		return -1;
@@ -1049,201 +892,3 @@ int i_enum_query_2(struct sip_msg* _msg, char* _suffix, char* _service)
 
 	return do_query(_msg, string, name, service);
 }
-
-
-
-/******************* FQUERY *******************/
-
-
-/*
- * Call enum_pv_query_3 with pv arg, module parameter suffix,
- * and default service.
- */
-int enum_pv_query_1(struct sip_msg* _msg, char* _sp)
-{
-    return enum_pv_query_3(_msg, _sp, (char *)(&suffix), (char *)(&service));
-}
-
-/*
- * Call enum_pv_query_3 with pv and suffix args and default service.
- */
-int enum_pv_query_2(struct sip_msg* _msg, char* _sp, char* _suffix)
-{
-    return enum_pv_query_3(_msg, _sp, _suffix, (char *)(&service));
-}
-
-/*
- * See documentation in README file.
- */
-
-int enum_pv_query_3(struct sip_msg* _msg, char* _sp, char* _suffix,
-		    char* _service)
-{
-	char *user_s;
-	int user_len, i, j, first;
-	char name[MAX_DOMAIN_SIZE];
-	char uri[MAX_URI_SIZE];
-	char new_uri[MAX_URI_SIZE];
-	unsigned int priority, curr_prio;
-	qvalue_t q;
-	char tostring[17];
-	struct rdata* head;
-	struct rdata* l;
-	struct naptr_rdata* naptr;
-	str pattern, replacement, result, new_result;
-	str *suffix, *service;
-	char string[17];
-	pv_spec_t *sp;
-	pv_value_t pv_val;
-
-	sp = (pv_spec_t *)_sp;
-	suffix = (str*)_suffix;
-	service = (str*)_service;
-
-	/*
-	 *  Get R-URI user to tostring
-	 */
-	if (parse_sip_msg_uri(_msg) < 0) {
-		LM_ERR("R-URI parsing failed\n");
-		return -1;
-	}
-
-	user_s = _msg->parsed_uri.user.s;
-	user_len = _msg->parsed_uri.user.len;
-
-	memcpy(&(tostring[0]), user_s, user_len);
-	tostring[user_len] = (char)0;
-
-	/*
-	 * Get E.164 number from pseudo variable
-         */
-	if (sp && (pv_get_spec_value(_msg, sp, &pv_val) == 0)) {
-	    if (pv_val.flags & PV_VAL_STR) {
-		if (pv_val.rs.len == 0 || pv_val.rs.s == NULL) {
-		    LM_DBG("Missing E.164 number\n");
-		    return -1;
-		}
-	    } else {
-		LM_DBG("Pseudo variable value is not string\n");
-		return -1;
-	}
-	} else {
-	    LM_DBG("Cannot get pseudo variable value\n");
-	    return -1;
-	}
-	if (is_e164(&(pv_val.rs)) == -1) {
-	    LM_ERR("pseudo variable does not contain an E164 number\n");
-	    return -1;
-	}
-
-	user_s = pv_val.rs.s;
-	user_len = pv_val.rs.len;
-
-	memcpy(&(string[0]), user_s, user_len);
-	string[user_len] = (char)0;
-
-	j = 0;
-	for (i = user_len - 1; i > 0; i--) {
-		name[j] = user_s[i];
-		name[j + 1] = '.';
-		j = j + 2;
-	}
-
-	memcpy(name + j, suffix->s, suffix->len + 1);
-
-	head = get_record(name, T_NAPTR);
-
-	if (head == 0) {
-		LM_DBG("No NAPTR record found for %s.\n", name);
-		return -1;
-	}
-
-	naptr_sort(&head);
-
-	q = MAX_Q - 10;
-	curr_prio = 0;
-	first = 1;
-
-	for (l = head; l; l = l->next) {
-
-		if (l->type != T_NAPTR) continue; /*should never happen*/
-		naptr = (struct naptr_rdata*)l->rdata;
-		if (naptr == 0) {
-			LM_ERR("Null rdata in DNS response\n");
-			continue;
-		}
-
-		LM_DBG("ENUM query on %s: order %u, pref %u, flen %u, flags "
-		       "'%.*s', slen %u, services '%.*s', rlen %u, "
-		       "regexp '%.*s'\n",
-		       name, naptr->order, naptr->pref,
-		    naptr->flags_len, (int)(naptr->flags_len), ZSW(naptr->flags),
-		    naptr->services_len,
-		    (int)(naptr->services_len), ZSW(naptr->services), naptr->regexp_len,
-		    (int)(naptr->regexp_len), ZSW(naptr->regexp));
-
-		if (sip_match(naptr, service) == 0) continue;
-
-		if (parse_naptr_regexp(&(naptr->regexp[0]), naptr->regexp_len,
-				       &pattern, &replacement) < 0) {
-			LM_ERR("Parsing of NAPTR regexp failed\n");
-			continue;
-		}
-		result.s = &(uri[0]);
-		result.len = MAX_URI_SIZE;
-		/* Avoid making copies of pattern and replacement */
-		pattern.s[pattern.len] = (char)0;
-		replacement.s[replacement.len] = (char)0;
-		if (reg_replace(pattern.s, replacement.s, &(tostring[0]),
-				&result) < 0) {
-			pattern.s[pattern.len] = '!';
-			replacement.s[replacement.len] = '!';
-			LM_ERR("Regexp replace failed\n");
-			continue;
-		}
-		LM_DBG("Resulted in replacement: '%.*s'\n",
-		       result.len, ZSW(result.s));
-		pattern.s[pattern.len] = '!';
-		replacement.s[replacement.len] = '!';
-
-		if (param.len > 0) {
-			if (result.len + param.len > MAX_URI_SIZE - 1) {
-				LM_ERR("URI is too long\n");
-				continue;
-			}
-			new_result.s = &(new_uri[0]);
-			new_result.len = MAX_URI_SIZE;
-			if (add_uri_param(&result, &param, &new_result) == 0) {
-				LM_ERR("Parsing of URI <%.*s> failed\n",
-				       result.len, result.s);
-				continue;
-			}
-			if (new_result.len > 0) {
-				result = new_result;
-			}
-		}
-
-		if (first) {
-			if (set_ruri(_msg, &result) == -1) {
-				goto done;
-			}
-			set_ruri_q(_msg, q);
-			first = 0;
-			curr_prio = ((naptr->order) << 16) + naptr->pref;
-		} else {
-			priority = ((naptr->order) << 16) + naptr->pref;
-			if (priority > curr_prio) {
-				q = q - 10;
-				curr_prio = priority;
-			}
-			if (append_branch(_msg, &result, 0, 0, q, 0, 0) == -1) {
-				goto done;
-			}
-		}
-	}
-
-done:
-	free_rdata_list(head);
-	return first ? -1 : 1;
-}
-
