@@ -683,7 +683,7 @@ static int media_exchange_from_uri(struct sip_msg *msg, str *uri, int leg,
 			body = media_exchange_get_offer_sdp(ctx, dlg,
 					DLG_MEDIA_SESSION_LEG(dlg, leg), &release);
 		} else {
-			sbody = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_LEG(dlg, other_leg(dlg, leg)));
+			sbody = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_OTHER_LEG(dlg, leg));
 			body = &sbody;
 		}
 	}
@@ -1655,7 +1655,7 @@ static mi_response_t *mi_media_exchange_from_call_to_uri(const mi_params_t *para
 			pbody = media_exchange_get_offer_sdp(ctx, dlg,
 					DLG_MEDIA_SESSION_LEG(dlg, media_leg), &release);
 		} else {
-			body = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_LEG(dlg, media_leg));
+			body = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_OTHER_LEG(dlg, media_leg));
 			pbody = &body;
 		}
 	} else {
