@@ -39,6 +39,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <fnmatch.h>
 
 #include "ratelimit.h"
 
@@ -55,14 +56,13 @@ static rl_algo_t get_rl_algo(str);
 /* big hash table */
 rl_big_htable rl_htable;
 
-/* feedback algorithm */
-static int *rl_feedback_limit;
-
 static cachedb_funcs cdbf;
 static cachedb_con *cdbc = 0;
 
 int rl_buffer_th = RL_BUF_THRESHOLD;
 
+static str pipe_repl_cap = str_init("ratelimit-pipe-repl");
+
 /* returnes the idex of the pipe in our hash */
 #define RL_GET_INDEX(_n)		core_hash(&(_n), NULL, rl_htable.size);
 
@@ -83,7 +83,12 @@ int rl_buffer_th = RL_BUF_THRESHOLD;
 
 /* returns true if the pipe should use cachedb interface */
 #define RL_USE_CDB(_p) \
-	(cdbc && (_p)->algo!=PIPE_ALGO_NETWORK && (_p)->algo!=PIPE_ALGO_FEEDBACK)
+	(cdbc && (_p)->algo!=PIPE_ALGO_NETWORK && \
+	 (_p)->algo!=PIPE_ALGO_FEEDBACK && \
+	 (_p)->flags&RL_PIPE_REPLICATE_CACHE)
+
+#define RL_USE_BIN(_p) \
+	 ((_p)->flags&RL_PIPE_REPLICATE_BIN)
 
 
 
@@ -167,8 +172,8 @@ static int rl_get_counter(str *name, rl_pipe_t * pipe)
 int init_cachedb(str * db_url)
 {
 	if (cachedb_bind_mod(db_url, &cdbf) < 0) {
-		LM_ERR("cannot bind functions for db_url %.*s\n",
-			db_url->len, db_url->s);
+		LM_ERR("cannot bind functions for db_url %s\n",
+			db_url_escape(db_url));
 		return -1;
 	}
 	if (!CACHEDB_CAPABILITY(&cdbf,
@@ -178,7 +183,7 @@ int init_cachedb(str * db_url)
 	}
 	cdbc = cdbf.init(db_url);
 	if (!cdbc) {
-		LM_ERR("cannot connect to db_url %.*s\n", db_url->len, db_url->s);
+		LM_ERR("cannot connect to db_url %s\n", db_url_escape(db_url));
 		return -1;
 	}
 	/* guessing that the name is not larger than 32 */
@@ -231,7 +236,7 @@ int init_rl_table(unsigned int size)
 	/* resolve the default algorithm */
 	rl_default_algo = get_rl_algo(rl_default_algo_s);
 	if (rl_default_algo == PIPE_ALGO_NOP) {
-		LM_ERR("unknown algoritm <%.*s>\n", rl_default_algo_s.len,
+		LM_ERR("unknown algorithm <%.*s>\n", rl_default_algo_s.len,
 			rl_default_algo_s.s);
 		return -1;
 	}
@@ -307,37 +312,107 @@ static str * get_rl_algo_name(rl_algo_t algo)
 	return NULL;
 }
 
-int w_rl_check_2(struct sip_msg *_m, char *_n, char *_l)
+rl_pipe_t *rl_create_pipe(int limit, rl_algo_t algo, str *name, unsigned flags)
 {
-	return w_rl_check_3(_m, _n, _l, NULL);
+	rl_pipe_t *pipe;
+	int size = sizeof(rl_pipe_t);
+
+	if (algo == PIPE_ALGO_NOP)
+		algo = rl_default_algo;
+
+	if (algo == PIPE_ALGO_HISTORY)
+		size += (rl_window_size * 1000) / rl_slot_period * sizeof(long int);
+
+#ifdef RL_DEBUG_PIPES
+	size += name->len;
+#endif
+	if (flags & RL_PIPE_REPLICATE_CACHE) {
+		if (!cdbc) {
+			LM_WARN("cachedb replication not configured! ignoring...\n");
+			flags &= ~RL_PIPE_REPLICATE_CACHE;
+		}
+		if (algo == PIPE_ALGO_NETWORK ||
+				algo == PIPE_ALGO_FEEDBACK) {
+			LM_WARN("cachedb replication not possible for "
+					"NETWORK and FEEDBACK algorithms!\n");
+			flags &= ~RL_PIPE_REPLICATE_CACHE;
+		}
+	}
+	if (flags & RL_PIPE_REPLICATE_BIN) {
+		if (!rl_repl_cluster) {
+			LM_WARN("clusterer replication not configured! ignoring...\n");
+			flags &= ~RL_PIPE_REPLICATE_BIN;
+		}
+	}
+
+	pipe = shm_malloc(size);
+	if (!pipe) {
+		LM_ERR("no more shm memory!\n");
+		return NULL;
+	}
+	memset(pipe, 0, size);
+
+	pipe->algo = algo;
+	pipe->limit = limit;
+	pipe->flags = flags;
+
+	if (algo == PIPE_ALGO_HISTORY) {
+		pipe->rwin.window = (long int *)(pipe + 1);
+		pipe->rwin.window_size = (rl_window_size * 1000) / rl_slot_period;
+		/* everything else is already cleared */
+	}
+
+#ifdef RL_DEBUG_PIPES
+	pipe->name.s = ((char *)pipe) + size - name->len;
+	memcpy(pipe->name.s, name->s, name->len);
+	pipe->name.len = name->len;
+#endif
+	return pipe;
 }
 
-int w_rl_check_3(struct sip_msg *_m, char *_n, char *_l, char *_a)
+static inline void parse_pipe_name(str *name, str *pipe_name, unsigned *flags)
 {
-	str name;
-	int limit = 0, ret = 1, should_update = 0;
-	str algorithm;
+	char *c;
+	*flags = 0;
+	*pipe_name = *name;
+	if (name->len < 2)
+		return;
+	for (c = &name->s[name->len - 1]; c > name->s; c--) {
+		switch (*c) {
+			case 'r':
+				*flags |= RL_PIPE_REPLICATE_CACHE;
+				break;
+			case 'b':
+				*flags |= RL_PIPE_REPLICATE_BIN;
+				break;
+			case '/':
+				if (*flags) {
+					pipe_name->len = c - pipe_name->s;
+					trim(pipe_name);
+				}
+				/* fall back */
+			default:
+				if (pipe_name->len == name->len)
+					*flags = 0;
+				/* not flags - something else at the end */
+				return;
+		}
+	}
+}
+
+int w_rl_check(struct sip_msg *_m, str *name, int *limit, str *algorithm)
+{
+	int ret = 1, should_update = 0;
 	unsigned int hash_idx;
 	rl_pipe_t **pipe;
+	str pipe_name;
+	unsigned flags;
 
 	rl_algo_t algo = -1;
 
-	/* retrieve and check parameters */
-	if (!_n || !_l) {
-		LM_ERR("invalid parameters\n");
-		goto end;
-	}
-	if (fixup_get_svalue(_m, (gparam_p) _n, &name) < 0) {
-		LM_ERR("cannot retrieve identifier\n");
-		goto end;
-	}
-	if (fixup_get_ivalue(_m, (gparam_p) _l, &limit) < 0) {
-		LM_ERR("cannot retrieve limit\n");
-		goto end;
-	}
-	algorithm.s = 0;
-	if (!_a || fixup_get_svalue(_m, (gparam_p) _a, &algorithm) < 0 ||
-		(algo = get_rl_algo(algorithm)) == PIPE_ALGO_NOP) {
+	parse_pipe_name(name, &pipe_name, &flags);
+
+	if (!algorithm || (algo = get_rl_algo(*algorithm)) == PIPE_ALGO_NOP) {
 		algo = PIPE_ALGO_NOP;
 	}
 
@@ -345,30 +420,30 @@ int w_rl_check_3(struct sip_msg *_m, char *_n, char *_l, char *_a)
 	if (algo == PIPE_ALGO_FEEDBACK) {
 		lock_get(rl_lock);
 		if (*rl_feedback_limit) {
-			if (*rl_feedback_limit != limit) {
+			if (*rl_feedback_limit != *limit) {
 				LM_WARN("FEEDBACK limit should be the same for all pipes, but"
 					" new limit %d differs - setting to %d\n",
-					limit, *rl_feedback_limit);
-				limit = *rl_feedback_limit;
+					*limit, *rl_feedback_limit);
+				*limit = *rl_feedback_limit;
 			}
 		} else {
-			if (limit <= 0 || limit >= 100) {
+			if (*limit <= 0 || *limit >= 100) {
 				LM_ERR("invalid limit for FEEDBACK algorithm "
 					"(must be between 0 and 100)\n");
 				lock_release(rl_lock);
 				goto end;
 			}
-			*rl_feedback_limit = limit;
-			pid_setpoint_limit(limit);
+			*rl_feedback_limit = *limit;
+			pid_setpoint_limit(*limit);
 		}
 		lock_release(rl_lock);
 	}
 
-	hash_idx = RL_GET_INDEX(name);
+	hash_idx = RL_GET_INDEX(pipe_name);
 	RL_GET_LOCK(hash_idx);
 
 	/* try to get the value */
-	pipe = RL_GET_PIPE(hash_idx, name);
+	pipe = RL_GET_PIPE(hash_idx, pipe_name);
 	if (!pipe) {
 		LM_ERR("cannot get the index\n");
 		goto release;
@@ -376,40 +451,37 @@ int w_rl_check_3(struct sip_msg *_m, char *_n, char *_l, char *_a)
 
 	if (!*pipe) {
 		/* allocate new pipe */
-		*pipe = shm_malloc(sizeof(rl_pipe_t) +
-				/* memory for the window */
-				(rl_window_size*1000) / rl_slot_period * sizeof(long int));
-		if (!*pipe) {
-			LM_ERR("no more shm memory\n");
+		if (!(*pipe = rl_create_pipe(*limit, algo, &pipe_name, flags)))
 			goto release;
-		}
-		memset(*pipe, 0, sizeof(rl_pipe_t));
+
 		LM_DBG("Pipe %.*s doesn't exist, but was created %p\n",
-				name.len, name.s, *pipe);
-		if (algo == PIPE_ALGO_NETWORK)
+				pipe_name.len, pipe_name.s, *pipe);
+		if ((*pipe)->algo == PIPE_ALGO_NETWORK)
 			should_update = 1;
-		(*pipe)->algo = (algo == PIPE_ALGO_NOP) ? rl_default_algo : algo;
-		(*pipe)->rwin.window = (long int *)((*pipe) + 1);
-		(*pipe)->rwin.window_size   = rl_window_size * 1000 / rl_slot_period;
-		memset((*pipe)->rwin.window, 0,
-				(*pipe)->rwin.window_size * sizeof(long int));
 	} else {
-		LM_DBG("Pipe %.*s found: %p - last used %lu\n",
-			name.len, name.s, *pipe, (*pipe)->last_used);
+		LM_DBG("Pipe %.*s found: %p - last used %lld\n",
+			pipe_name.len, pipe_name.s, *pipe, (long long)(*pipe)->last_used);
 		if (algo != PIPE_ALGO_NOP && (*pipe)->algo != algo) {
 			LM_WARN("algorithm %d different from the initial one %d for pipe "
-				"%.*s", algo, (*pipe)->algo, name.len, name.s);
+				"%.*s\n", algo, (*pipe)->algo, pipe_name.len, pipe_name.s);
+		}
+		/* update the limit */
+		(*pipe)->limit = *limit;
+		if ((*pipe)->flags != flags) {
+			LM_DBG("updating %.*s pipe flags from %x to %x\n",
+					pipe_name.len, pipe_name.s, (*pipe)->flags,
+					((*pipe)->flags | flags));
+			(*pipe)->flags |= flags; /* also update the flags */
 		}
 	}
 
-	/* set/update the limit */
-	(*pipe)->limit = limit;
-
 	/* set the last used time */
 	(*pipe)->last_used = time(0);
+	/* set the last 'local' used time: */
+	(*pipe)->last_local_used = time(0);
 	if (RL_USE_CDB(*pipe)) {
 		/* release the counter for a while */
-		if (rl_change_counter(&name, *pipe, 1) < 0) {
+		if (rl_change_counter(&pipe_name, *pipe, 1) < 0) {
 			LM_ERR("cannot increase counter\n");
 			goto release;
 		}
@@ -419,7 +491,7 @@ int w_rl_check_3(struct sip_msg *_m, char *_n, char *_l, char *_a)
 
 	ret = rl_pipe_check(*pipe);
 	LM_DBG("Pipe %.*s counter:%d load:%d limit:%d should %sbe blocked (%p)\n",
-		name.len, name.s, (*pipe)->counter, (*pipe)->load,
+		pipe_name.len, pipe_name.s, (*pipe)->counter, (*pipe)->load,
 		(*pipe)->limit, ret == 1 ? "NOT " : "", *pipe);
 
 
@@ -435,7 +507,7 @@ end:
 }
 
 /* timer housekeeping, invoked each timer interval to reset counters */
-void rl_timer(unsigned int ticks, void *param)
+void rl_timer(utime_t uticks, void *param)
 {
 	unsigned int i = 0;
 	map_iterator_t it, del;
@@ -481,7 +553,12 @@ void rl_timer(unsigned int ticks, void *param)
 				goto next_pipe;
 			}
 			/* check to see if it is expired */
-			if ((*pipe)->last_used + rl_expire_time < now) {
+			if (((*pipe)->last_local_used + rl_expire_time < now) &&
+				/* We shall wait a while until we make sure all destinations
+				 * have sent us replicated packets - after that, we can delete
+				 * the pipe; the condition is always true if pipe is not
+				 * replicated */
+				((*pipe)->last_used + (RL_USE_BIN(*pipe)?rl_repl_timer_expire:0) < now)) {
 				/* this pipe is engaged in a transaction */
 				del = it;
 				if (iterator_next(&it) < 0)
@@ -516,7 +593,8 @@ void rl_timer(unsigned int ticks, void *param)
 				case PIPE_ALGO_RED:
 					if ((*pipe)->limit && rl_timer_interval)
 						(*pipe)->load = (*pipe)->counter /
-						((*pipe)->limit * rl_timer_interval);
+						((*pipe)->limit *
+							(rl_limit_per_interval ? 1 : rl_timer_interval));
 					break;
 				default:
 					break;
@@ -540,34 +618,20 @@ next_map:
 	}
 }
 
-struct rl_param_t {
-	int counter;
-	struct mi_node * node;
-	struct mi_root * root;
-};
-
 static int rl_map_print(void *param, str key, void *value)
 {
-	struct mi_attr* attr;
-	char* p;
-	int len;
-	struct rl_param_t * rl_param = (struct rl_param_t *) param;
-	struct mi_node * rpl;
 	rl_pipe_t *pipe = (rl_pipe_t *) value;
-	struct mi_node * node;
 	str *alg;
+	mi_item_t *pipe_item = (mi_item_t *)param;
 
 	if (!pipe) {
 		LM_ERR("invalid pipe value\n");
 		return -1;
 	}
-
-	if (!rl_param || !rl_param->node || !rl_param->root) {
-		LM_ERR("no reply node\n");
+	if (!pipe_item) {
+		LM_ERR("no mi item\n");
 		return -1;
 	}
-	rpl = rl_param->node;
-
 	if (!key.len || !key.s) {
 		LM_ERR("no key found\n");
 		return -1;
@@ -577,10 +641,7 @@ static int rl_map_print(void *param, str key, void *value)
 	if (pipe->algo == PIPE_ALGO_NOP)
 		return 0;
 
-	if (!(node = add_mi_node_child(rpl, 0, "PIPE", 4, 0, 0)))
-		return -1;
-
-	if (!(attr = add_mi_attr(node, MI_DUP_VALUE, "id", 2, key.s, key.len)))
+	if (add_mi_string(pipe_item, MI_SSTR("id"), key.s, key.len) < 0)
 		return -1;
 
 	if (!(alg = get_rl_algo_name(pipe->algo))) {
@@ -588,36 +649,63 @@ static int rl_map_print(void *param, str key, void *value)
 		return -1;
 	}
 
-	if (!(attr = add_mi_attr(node, MI_DUP_VALUE, "algorithm", 9,
-		alg->s, alg->len)))
+	if (add_mi_string(pipe_item, MI_SSTR("algorithm"), alg->s, alg->len) < 0)
 		return -1;
 
-
-	p = int2str((unsigned long) (pipe->limit), &len);
-	if (!(attr = add_mi_attr(node, MI_DUP_VALUE, "limit", 5, p, len)))
+	if (add_mi_number(pipe_item, MI_SSTR("limit"), pipe->limit) < 0)
 		return -1;
 
-	p = int2str((unsigned long)(pipe->last_counter), &len);
-	if (!(attr = add_mi_attr(node, MI_DUP_VALUE, "counter", 7, p, len)))
+	if (add_mi_number(pipe_item, MI_SSTR("counter"), pipe->last_counter) < 0)
 		return -1;
 
-	if ((++rl_param->counter % 50) == 0) {
-		LM_DBG("flush mi tree - number %d\n", rl_param->counter);
-		flush_mi_tree(rl_param->root);
-	}
-
 	return 0;
 }
 
-int rl_stats(struct mi_root *rpl_tree, str * value)
+static int rl_map_print_array(void *extras, str key, void *value)
 {
-	rl_pipe_t **pipe;
-	struct rl_param_t param;
-	int i;
+	mi_item_t *pipe_item = add_mi_object((mi_item_t *)(((void**)extras)[0]),
+		NULL, 0);
+	if (!pipe_item)
+		return -1;
+	return rl_map_print(pipe_item, key, value );
+}
 
-	memset(&param, 0, sizeof(struct rl_param_t));
-	param.node = &rpl_tree->node;
-	param.root = rpl_tree;
+static int rl_map_print_array_filter(void *extras, str key, void *value)
+{
+	static str nt_key = {NULL,0};
+	mi_item_t *pipe_item;
+	char *filter_s = (char*)(((void**)extras)[1]);
+	int filter_out = (int)(long)(((void**)extras)[2]);
+
+	/* make the key null terminated */
+	if (pkg_str_extend( &nt_key, key.len+1))
+		return -1;
+	memcpy( nt_key.s, key.s, key.len);
+	nt_key.s[key.len] = 0;
+
+	if ( fnmatch( filter_s, nt_key.s, 0)!=0 ) {
+		if (filter_out==0)
+			return 0;
+	} else {
+		if (filter_out==1)
+			return 0;
+	}
+
+	pipe_item = add_mi_object((mi_item_t *)(((void**)extras)[0]), NULL, 0);
+	if (!pipe_item)
+		return -1;
+
+	return rl_map_print(pipe_item, key, value );
+}
+
+int rl_stats(mi_item_t *resp_obj, str * value, str *filter, int filter_out)
+{
+	mi_item_t *pipe_item, *pipe_arr;
+	rl_pipe_t **pipe;
+	void *extras[3];
+	char *filter_s;
+	process_each_func func;
+	int i;
 
 	if (value && value->s && value->len) {
 		i = RL_GET_INDEX(*value);
@@ -625,9 +713,13 @@ int rl_stats(struct mi_root *rpl_tree, str * value)
 		pipe = RL_FIND_PIPE(i, *value);
 		if (!pipe || !*pipe) {
 			LM_DBG("pipe %.*s not found\n", value->len, value->s);
-			goto error;
+			RL_RELEASE_LOCK(i);
+			return 1;
 		}
-		if (rl_map_print(&param, *value, *pipe)) {
+		pipe_item = add_mi_object(resp_obj, MI_SSTR("Pipe"));
+		if (!pipe_item)
+			goto error;
+		if (rl_map_print(pipe_item, *value, *pipe)) {
 			LM_ERR("cannot print value for key %.*s\n",
 				value->len, value->s);
 			goto error;
@@ -635,9 +727,28 @@ int rl_stats(struct mi_root *rpl_tree, str * value)
 		RL_RELEASE_LOCK(i);
 	} else {
 		/* iterate through each map */
+		pipe_arr = add_mi_array(resp_obj, MI_SSTR("Pipes"));
+		if (!pipe_arr)
+			return -1;
+		extras[0] = pipe_arr;
+		if (filter && filter->s && filter->len) {
+			// make filter NULL terminated
+			filter_s = (char*)pkg_malloc( filter->len+1 );
+			if (!filter_s)
+				return -1;
+			memcpy( filter_s, filter->s, filter->len);
+			filter_s[filter->len] = 0;
+			extras[1] = filter_s;
+			extras[2] = (void*)(long)filter_out;
+			func = rl_map_print_array_filter;
+		} else {
+			func = rl_map_print_array;
+		}
 		for (i = 0; i < rl_htable.size; i++) {
+			if (map_size(rl_htable.maps[i]) == 0)
+				continue;
 			RL_GET_LOCK(i);
-			if (map_for_each(rl_htable.maps[i], rl_map_print, &param)) {
+			if (map_for_each(rl_htable.maps[i], func, (void*)extras)) {
 				LM_ERR("cannot print values\n");
 				goto error;
 			}
@@ -650,87 +761,6 @@ error:
 	return -1;
 }
 
-static int bin_status_aux(struct mi_node *root, clusterer_node_t *nodes, int type, int cluster_id)
-{
-	clusterer_node_t *d;
-	struct mi_node *node = NULL;
-	struct mi_attr* attr;
-	str cluster_id_s;
-	str machine_id_s;
-	str state;
-
-	for (d = nodes; d; d = d->next) {
-		cluster_id_s.s = int2str(cluster_id, &cluster_id_s.len);
-		node = add_mi_node_child(root, MI_DUP_VALUE, "Cluster ID", 10,
-			cluster_id_s.s, cluster_id_s.len);
-		if (node == NULL) goto error;
-
-		machine_id_s.s = int2str(d->machine_id, &machine_id_s.len);
-		attr = add_mi_attr(node, MI_DUP_VALUE, "Machine ID", 10,
-			machine_id_s.s, machine_id_s.len);
-		if (attr == NULL) goto error;
-
-		state.s = int2str(d->state, &state.len);
-		attr = add_mi_attr(node, MI_DUP_VALUE, "STATE", 5,
-			state.s, state.len);
-		if (attr == NULL) goto error;
-
-		if (d->description.s)
-			attr = add_mi_attr(node, MI_DUP_VALUE, "DESCRIPTION", 11,
-			d->description.s, d->description.len);
-		else
-			attr = add_mi_attr(node, MI_DUP_VALUE, "DESCRIPTION", 11,
-			"none", 4);
-		if (attr == NULL) goto error;
-
-		if (type)
-			attr = add_mi_attr(node, MI_DUP_VALUE, "TYPE", 4,
-			"server", 6);
-		else
-			attr = add_mi_attr(node, MI_DUP_VALUE, "TYPE", 4,
-			"client", 6);
-		if (attr == NULL) goto error;
-	}
-
-	return 0;
-error:
-	return -1;
-}
-
-int rl_bin_status(struct mi_root *rpl_tree)
-{
-	clusterer_node_t *nodes;
-	struct mi_node *root = NULL;
-
-	if (rpl_tree == NULL)
-		return -1;
-
-	root = &rpl_tree->node;
-	if (rl_repl_cluster) {
-		nodes = clusterer_api.get_nodes(rl_repl_cluster, PROTO_BIN);
-		if (nodes == NULL)
-			return -1;
-		if (bin_status_aux(root, nodes, 1, rl_repl_cluster) < 0)
-			goto error;
-		clusterer_api.free_nodes(nodes);
-	}
-
-	if (accept_repl_pipes) {
-		nodes = clusterer_api.get_nodes(accept_repl_pipes, PROTO_BIN);
-		if (nodes == NULL)
-			return -1;
-		if (bin_status_aux(root, nodes, 0, accept_repl_pipes) < 0)
-			goto error;
-		clusterer_api.free_nodes(nodes);
-	}
-
-
-	return 0;
-error:
-	clusterer_api.free_nodes(nodes);
-	return -1;
-}
-
 int w_rl_set_count(str key, int val)
 {
 	unsigned int hash_idx;
@@ -752,6 +782,8 @@ int w_rl_set_count(str key, int val)
 			LM_ERR("cannot decrease counter\n");
 			goto release;
 		}
+	} else if ((*pipe)->algo == PIPE_ALGO_HISTORY) {
+		hist_set_count(*pipe, val);
 	} else {
 		if (val && (val + (*pipe)->counter >= 0)) {
 			(*pipe)->counter += val;
@@ -770,27 +802,21 @@ release:
 	return ret;
 }
 
-static inline int w_rl_change_counter(struct sip_msg *_m, char *_n, int dec)
+static inline int w_rl_change_counter(struct sip_msg *_m, str *name, int dec)
 {
-	str name;
-
-	if (!_n || fixup_get_svalue(_m, (gparam_p) _n, &name) < 0) {
-		LM_ERR("cannot retrieve identifier\n");
-		return -1;
-	}
-	if (w_rl_set_count(name, dec)) {
-		LM_ERR("cannot find any pipe named %.*s\n", name.len, name.s);
+	if (w_rl_set_count(*name, dec)) {
+		LM_ERR("cannot find any pipe named %.*s\n", name->len, name->s);
 		return -1;
 	}
 	return 1;
 }
 
-int w_rl_dec(struct sip_msg *_m, char *_n)
+int w_rl_dec(struct sip_msg *_m, str *_n)
 {
 	return w_rl_change_counter(_m, _n, -1);
 }
 
-int w_rl_reset(struct sip_msg *_m, char *_n)
+int w_rl_reset(struct sip_msg *_m, str *_n)
 {
 	return w_rl_change_counter(_m, _n, 0);
 }
@@ -826,55 +852,45 @@ error:
 
 
 
-void rl_rcv_bin(int packet_type, struct receive_info *ri, int server_id)
+void rl_rcv_bin(bin_packet_t *packet)
 {
 	rl_algo_t algo;
 	int limit;
 	int counter;
 	str name;
-	char *ip;
-	unsigned short port;
 	rl_pipe_t **pipe;
-	unsigned int hash_idx;
+	unsigned int hash_idx, flags;
 	time_t now;
 	rl_repl_counter_t *destination;
 
-	if (packet_type == SERVER_TEMP_DISABLED) {
- 		get_su_info(&ri->src_su.s, ip, port);
-		LM_WARN("server: %s:%hu temporary disabled\n", ip, port);
- 		return;
- 	}
-
-	if (packet_type == SERVER_TIMEOUT) {
-		LM_INFO("server with clustererer id %d timeout\n", server_id);
+	if (packet->type != RL_PIPE_COUNTER) {
+		LM_WARN("Invalid binary packet command: %d (from node: %d in cluster: %d)\n",
+			packet->type, packet->src_id, rl_repl_cluster);
 		return;
 	}
 
-	if(get_bin_pkg_version() != BIN_VERSION){
-		LM_ERR("incompatible bin protocol version\n");
-		return;
-	}
-
-	if (packet_type != RL_PIPE_COUNTER)
-		return;
-
 	now = time(0);
 
 	for (;;) {
-		if (bin_pop_str(&name) == 1)
+		if (bin_pop_str(packet, &name) == 1)
 			break; /* pop'ed all pipes */
 
-		if (bin_pop_int(&algo) < 0) {
+		if (bin_pop_int(packet, &flags) < 0) {
+			LM_ERR("cannot pop pipe's flags\n");
+			return;
+		}
+
+		if (bin_pop_int(packet, &algo) < 0) {
 			LM_ERR("cannot pop pipe's algorithm\n");
 			return;
 		}
 
-		if (bin_pop_int(&limit) < 0) {
+		if (bin_pop_int(packet, &limit) < 0) {
 			LM_ERR("cannot pop pipe's limit\n");
 			return;
 		}
 
-		if (bin_pop_int(&counter) < 0) {
+		if (bin_pop_int(packet, &counter) < 0) {
 			LM_ERR("cannot pop pipe's counter\n");
 			return;
 		}
@@ -890,20 +906,14 @@ void rl_rcv_bin(int packet_type, struct receive_info *ri, int server_id)
 		}
 
 		if (!*pipe) {
-			/* if the pipe does not exist, alocate it in case we need it later */
-			*pipe = shm_malloc(sizeof(rl_pipe_t));
-			if (!*pipe) {
-				LM_ERR("no more shm memory\n");
+			/* if the pipe does not exist, allocate it in case we need it later */
+			if (!(*pipe = rl_create_pipe(limit, algo, &name, flags)))
 				goto release;
-			}
-			memset(*pipe, 0, sizeof(rl_pipe_t));
 			LM_DBG("Pipe %.*s doesn't exist, but was created %p\n",
 				name.len, name.s, *pipe);
-			(*pipe)->algo = algo;
-			(*pipe)->limit = limit;
 		} else {
-			LM_DBG("Pipe %.*s found: %p - last used %lu\n",
-				name.len, name.s, *pipe, (*pipe)->last_used);
+			LM_DBG("Pipe %.*s found: %p - last used %lld\n",
+				name.len, name.s, *pipe, (long long)(*pipe)->last_used);
 			if ((*pipe)->algo != algo)
 				LM_WARN("algorithm %d different from the initial one %d for "
 				"pipe %.*s", algo, (*pipe)->algo, name.len, name.s);
@@ -918,9 +928,12 @@ void rl_rcv_bin(int packet_type, struct receive_info *ri, int server_id)
 		/* set the last used time */
 		(*pipe)->last_used = time(0);
 		/* set the destination's counter */
-		destination = find_destination(*pipe, server_id);
+		destination = find_destination(*pipe, packet->src_id);
+		if (!destination)
+			goto release;
 		destination->counter = counter;
 		destination->update = now;
+		RL_DBG(*pipe, "counter=%d id=%d", counter, packet->src_id);
 		RL_RELEASE_LOCK(hash_idx);
 	}
 	return;
@@ -933,7 +946,7 @@ release:
  * same as hist_check() in ratelimit.c but this one
  * only counts, no updates on the window ==> faster
  */
-static inline int hist_count(rl_pipe_t *pipe)
+static inline int ALLOW_UNUSED hist_count(rl_pipe_t *pipe)
 {
 	/* Window ELement*/
 	#define U2MILI(__usec__) (__usec__/1000)
@@ -982,46 +995,54 @@ static inline int hist_count(rl_pipe_t *pipe)
 
 int rl_repl_init(void)
 {
-
-	if (rl_buffer_th > (BUF_SIZE * 0.9)) {
-		LM_WARN("Buffer size too big %d - pipe information might get lost",
-			rl_buffer_th);
-		return -1;
-	}
-
-	if (accept_repl_pipes &&
-		clusterer_api.register_module("ratelimit", PROTO_BIN, rl_rcv_bin,
-		accept_repl_pipes_timeout, repl_pipes_auth_check, accept_repl_pipes) < 0) {
-		LM_ERR("Cannot register binary packet callback!\n");
+	if (rl_repl_cluster && clusterer_api.register_capability(&pipe_repl_cap,
+		rl_rcv_bin, NULL, rl_repl_cluster, 0, NODE_CMP_ANY) < 0) {
+		LM_ERR("Cannot register clusterer callback!\n");
 		return -1;
 	}
 
 	return 0;
 }
 
-static inline void rl_replicate(void)
+static inline void rl_replicate(bin_packet_t *packet)
 {
-	if (clusterer_api.send_to(rl_repl_cluster, PROTO_BIN) < 0) {
-		LM_INFO("ratelimit replicate failed\n");
- 	}
+	int rc;
+
+	rc = clusterer_api.send_all(packet, rl_repl_cluster);
+	switch (rc) {
+	case CLUSTERER_CURR_DISABLED:
+		LM_INFO("Current node is disabled in cluster: %d\n", rl_repl_cluster);
+		goto error;
+	case CLUSTERER_DEST_DOWN:
+		LM_INFO("All destinations in cluster: %d are down or probing\n",
+			rl_repl_cluster);
+		goto error;
+	case CLUSTERER_SEND_ERR:
+		LM_ERR("Error sending in cluster: %d\n", rl_repl_cluster);
+		goto error;
+	}
+
+	return;
+
+error:
+	LM_ERR("Failed to replicate ratelimit pipes\n");
 }
 
 void rl_timer_repl(utime_t ticks, void *param)
 {
-	static str module_name = str_init("ratelimit");
 	unsigned int i = 0;
 	map_iterator_t it;
 	rl_pipe_t **pipe;
 	str *key;
 	int nr = 0;
-	int ret;
-	int replicated = 0;
+	int ret = 0;
+	bin_packet_t packet;
+	time_t now = time(0);
 
-	if (bin_init(&module_name, RL_PIPE_COUNTER, BIN_VERSION) < 0) {
+	if (bin_init(&packet, &pipe_repl_cap, RL_PIPE_COUNTER, BIN_VERSION, 0) < 0) {
 		LM_ERR("cannot initiate bin buffer\n");
 		return;
 	}
-	bin_push_int(clusterer_api.get_my_id());
 
 	/* iterate through each map */
 	for (i = 0; i < rl_htable.size; i++) {
@@ -1038,7 +1059,11 @@ void rl_timer_repl(utime_t ticks, void *param)
 				goto next_pipe;
 			}
 			/* ignore cachedb replicated stuff */
-			if (RL_USE_CDB(*pipe))
+			if (!RL_USE_BIN(*pipe))
+				goto next_pipe;
+
+			/* do not replicate if about to expire */
+			if ((*pipe)->last_local_used + rl_expire_time < now)
 				goto next_pipe;
 
 			key = iterator_key(&it);
@@ -1047,48 +1072,55 @@ void rl_timer_repl(utime_t ticks, void *param)
 				goto next_pipe;
 			}
 
-			if (bin_push_str(key) < 0)
+			if (bin_push_str(&packet, key) < 0)
 				goto error;
 
-			if (bin_push_int((*pipe)->algo) < 0)
+			if (bin_push_int(&packet, (*pipe)->flags) < 0)
 				goto error;
 
-			if (bin_push_int((*pipe)->limit) < 0)
+			if (bin_push_int(&packet, (*pipe)->algo) < 0)
 				goto error;
 
-			if ((ret = bin_push_int((*pipe)->my_last_counter)) < 0)
+			if (bin_push_int(&packet, (*pipe)->limit) < 0)
+				goto error;
+
+			/*
+			 * for the SBT algorithm it is safe to replicate the current
+			 * counter, since it is always updating according to the window
+			 */
+			RL_DBG(*pipe, "replicate=%d", ((*pipe)->algo == PIPE_ALGO_HISTORY ?
+						 (*pipe)->counter : (*pipe)->my_last_counter));
+			if ((ret = bin_push_int(&packet,
+						((*pipe)->algo == PIPE_ALGO_HISTORY ?
+						 (*pipe)->counter : (*pipe)->my_last_counter))) < 0)
 				goto error;
 			nr++;
 
-			if (ret > rl_buffer_th) {
-				/* always replicate the pipe to "simulate" pinging */
-				rl_replicate();
-				replicated = 1;
-				if (bin_init(&module_name, RL_PIPE_COUNTER, BIN_VERSION) < 0) {
-					LM_ERR("cannot initiate bin buffer\n");
-					RL_RELEASE_LOCK(i);
-					return;
-				}
-				bin_push_int(clusterer_api.get_my_id());
-				nr = 0;
-			}
-
 next_pipe:
 			if (iterator_next(&it) < 0)
 				break;
 		}
 next_map:
 		RL_RELEASE_LOCK(i);
+		if (ret > rl_buffer_th) {
+			/* send the buffer */
+			if (nr)
+				rl_replicate(&packet);
+			bin_reset_back_pointer(&packet);
+			nr = 0;
+		}
 	}
 	/* if there is anything else to send, do it now */
-	if (!replicated)
-		rl_replicate();
+	if (nr)
+		rl_replicate(&packet);
+	bin_free_packet(&packet);
 	return;
 error:
 	LM_ERR("cannot add pipe info in buffer\n");
 	RL_RELEASE_LOCK(i);
 	if (nr)
-		rl_replicate();
+		rl_replicate(&packet);
+	bin_free_packet(&packet);
 }
 
 int rl_get_all_counters(rl_pipe_t *pipe)
@@ -1102,7 +1134,8 @@ int rl_get_all_counters(rl_pipe_t *pipe)
 		/* if the replication expired, reset its counter */
 		if ((d->update + rl_repl_timer_expire) < now)
 			d->counter = 0;
-		counter += d->counter;
+		else
+			counter += d->counter;
 	}
 	return counter + pipe->counter;
 }
@@ -1128,8 +1161,10 @@ int rl_get_counter_value(str *key)
 			LM_ERR("cannot get the counter's value\n");
 			goto release;
 		}
-	}
-	ret = rl_get_all_counters(*pipe);
+	} else if ((*pipe)->algo == PIPE_ALGO_HISTORY)
+		ret = hist_get_count(*pipe);
+	else
+		ret = rl_get_all_counters(*pipe);
 
 release:
 	RL_RELEASE_LOCK(hash_idx);
