@@ -53,6 +53,7 @@
 #include "../../msg_translator.h"
 #include "../../mod_fix.h"
 #include "../../trim.h"
+#include "../../lib/cJSON.h"
 
 #include "codecs.h"
 #include "list_hdr.h"
@@ -87,6 +88,7 @@ static int get_glob_headers_values(struct sip_msg* msg, str* pattern,pv_spec_t*
 static int remove_hf_match_f(struct sip_msg* msg, void* pattern, int is_regex);
 static int is_present_hf(struct sip_msg* msg, void* _match_hf);
 static int append_to_reply_f(struct sip_msg* msg, str* key);
+static int append_body_to_reply_f(struct sip_msg* msg, str* key);
 static int append_hf(struct sip_msg *msg, str *str1, void *str2);
 static int insert_hf(struct sip_msg *msg, str *str1, void *str2);
 static int append_urihf(struct sip_msg *msg, str *str1, str *str2);
@@ -100,6 +102,7 @@ static int add_body_part_f(struct sip_msg *msg, str *body, str *mime,
 static int get_updated_body_part_f(struct sip_msg *msg, int *type,pv_spec_t* out);
 static int is_audio_on_hold_f(struct sip_msg *msg);
 static int w_sip_validate(struct sip_msg *msg, void *flags, pv_spec_t* err_txt);
+static int w_sip_to_json(struct sip_msg *msg, pv_spec_t* out_json);
 
 static int fixup_parse_hname(void** param);
 
@@ -122,6 +125,10 @@ static const cmd_export_t cmds[]={
 		{CMD_PARAM_STR, 0, 0}, {0, 0, 0}},
 		REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ERROR_ROUTE},
 
+	{"append_body_to_reply",  (cmd_function)append_body_to_reply_f, {
+		{CMD_PARAM_STR, 0, 0}, {0, 0, 0}},
+		REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ERROR_ROUTE},
+
 	{"append_hf",        (cmd_function)append_hf, {
 		{CMD_PARAM_STR, 0, 0},
 		{CMD_PARAM_STR|CMD_PARAM_OPT, fixup_parse_hname, fixup_free_pkg},
@@ -298,6 +305,9 @@ static const cmd_export_t cmds[]={
 	        {CMD_PARAM_VAR, 0, 0},
 		{0, 0, 0}},
 		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
+	{"sip_to_json",     (cmd_function)w_sip_to_json, {
+		{CMD_PARAM_VAR, 0, 0}, {0, 0, 0}},
+		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|LOCAL_ROUTE},
 
 	{0,0,{{0,0,0}},0}
 };
@@ -542,6 +552,27 @@ static int append_to_reply_f(struct sip_msg* msg, str* key)
 }
 
 
+static int append_body_to_reply_f(struct sip_msg* msg, str* body)
+{
+	struct lump_rpl *l;
+
+	l = get_lump_rpl( msg, LUMP_RPL_BODY);
+	if (l) {
+		if (replace_lump_rpl(l, body->s, body->len, LUMP_RPL_BODY)!=0) {
+			LM_ERR("unable to replace existing body lump_rl\n");
+			return -1;
+		}
+	} else {
+		if (add_lump_rpl( msg, body->s, body->len, LUMP_RPL_BODY)==NULL) {
+			LM_ERR("unable to create new body lump_rl\n");
+			return -1;
+		}
+	}
+
+	return 1;
+}
+
+
 /* add str1 to end of header or str1.r-uri.str2 */
 
 static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
@@ -1547,7 +1578,6 @@ static int check_hostname(str *domain)
 #define MAX_REASON 256
 
 enum sip_validation_failures {
-	SV_NO_ERROR=0,
 	SV_NO_MSG=-1,
 	SV_HDR_PARSE_ERROR=-2,
 	SV_NO_CALLID=-3,
@@ -1577,81 +1607,18 @@ enum sip_validation_failures {
 	SV_BAD_USERNAME=-27,
 	SV_FROM_USERNAME_ERROR=-28,
 	SV_TO_USERNAME_ERROR=-29,
-	SV_BAD_STAR_CONTACT=-30,
 	SV_GENERIC_FAILURE=-255
 };
 
-#define METHOD_WITH_CONTACT_HDR (METHOD_INVITE | METHOD_REGISTER | METHOD_UPDATE | METHOD_SUBSCRIBE | METHOD_REFER)
-
-static enum sip_validation_failures validate_contact_header(struct sip_msg *msg, enum request_method method, char reason[static MAX_REASON])
-{
-	struct hdr_field *ptr = NULL;
-	contact_t * contacts = NULL;
-	exp_body_t *expires_body = NULL;
-	struct sip_uri test_contacts;
-
-	if (method & METHOD_WITH_CONTACT_HDR) {
-		if (!msg->contact) {
-			strcpy(reason, "SIP message doesn't have 'Contact' header");
-			return SV_NO_CONTACT;
-		}
-
-		/* iterate through Contact headers */
-		for(ptr = msg->contact; ptr; ptr = ptr->sibling) {
-			/* parse Contact header */
-			if (!ptr->parsed && (parse_contact(ptr) < 0
-						|| !ptr->parsed)) {
-				strcpy(reason, "failed to parse 'Contact' header");
-				return SV_CONTACT_PARSE_ERROR;
-			}
-			contacts = ((contact_body_t*)ptr->parsed)->contacts;
-
-			if (contacts != NULL) {
-				/* iterate through URIs and check validty */
-				for (; contacts; contacts = contacts->next) {
-					if(parse_uri(contacts->uri.s, contacts->uri.len,
-								&test_contacts) < 0
-							|| test_contacts.host.len < 0) {
-						strcpy(reason, "failed to parse 'Contact' header");
-						return SV_CONTACT_PARSE_ERROR;
-					}
-				}
-			} else {
-				/* This is a star Contact header, valid only for REGISTER requests */
-				if (method == METHOD_REGISTER) {
-					if (msg->first_line.type == SIP_REPLY) {
-						strcpy(reason, "'Contact' header for REGISTER reply contains '*' only valid for REGISTER request");
-						return SV_BAD_STAR_CONTACT;
-					}
-
-					if (!msg->expires || (parse_expires(msg->expires) < 0 || !msg->expires->parsed)) {
-						strcpy(reason, "failed to parse 'Expires' header");
-						return SV_BAD_STAR_CONTACT;
-					}
-
-					expires_body = (exp_body_t*) msg->expires->parsed;
-
-					if (!expires_body || expires_body->val != 0) {
-						strcpy(reason, "'Expires' header greater than 0 for REGISTER 'Contact' header with '*' value");
-						return SV_BAD_STAR_CONTACT;
-					}
-				} else {
-					strcpy(reason, "'Contact' header for SIP message contains '*' only valid for REGISTER request");
-					return SV_BAD_STAR_CONTACT;
-				}
-			}
-		}
-	}
-
-	return SV_NO_ERROR;
-}
-
 static int w_sip_validate(struct sip_msg *msg, void *_flags, pv_spec_t* err_txt)
 {
-	unsigned int hdrs_len, status_code;
+	unsigned int hdrs_len;
 	unsigned long flags = (unsigned long)_flags;
 	int method;
 	str body;
+	struct hdr_field * ptr;
+	contact_t * contacts;
+	struct sip_uri test_contacts;
 	struct sip_uri *p_uri;
 	struct cseq_body * cbody;
 	pv_value_t pv_val;
@@ -1813,11 +1780,38 @@ static int w_sip_validate(struct sip_msg *msg, void *_flags, pv_spec_t* err_txt)
 				CHECK_HEADER("", maxforwards);
 			}
 
-			if (flags & SIP_PARSE_CONTACT) {
-				ret = validate_contact_header(msg, msg->REQ_METHOD, reason);
+			if (msg->REQ_METHOD == METHOD_INVITE) {
+				ret = SV_NO_CONTACT;
+				CHECK_HEADER("INVITE", contact);
+				if(flags & SIP_PARSE_CONTACT) {
+					/* iterate through Contact headers */
+					for(ptr = msg->contact; ptr; ptr = ptr->sibling) {
+						/* parse Contact header */
+						if(!ptr->parsed && (parse_contact(ptr) < 0
+									|| !ptr->parsed)) {
+							strcpy(reason, "failed to parse 'Contact' header");
+							ret = SV_CONTACT_PARSE_ERROR;
+							goto failed;
+						}
+						contacts = ((contact_body_t*)ptr->parsed)->contacts;
+						/* empty contacts header - something must be wrong */
+						if(contacts == NULL) {
+							strcpy(reason, "empty body for 'Contact' header");
+							ret = SV_CONTACT_PARSE_ERROR;
+							goto failed;
+						}
+						/* iterate through URIs and check validty */
+						for(; contacts; contacts = contacts->next) {
+							if(parse_uri(contacts->uri.s, contacts->uri.len,
+										&test_contacts) < 0
+									|| test_contacts.host.len < 0) {
+								strcpy(reason, "failed to parse 'Contact' header");
+								ret = SV_CONTACT_PARSE_ERROR;
+								goto failed;
+							}
+						}
+					}
 
-				if (ret != SV_NO_ERROR) {
-					goto failed;
 				}
 			}
 
@@ -1840,9 +1834,8 @@ static int w_sip_validate(struct sip_msg *msg, void *_flags, pv_spec_t* err_txt)
 				goto failed;
 			}
 			method = cbody->method_id;
-			status_code = msg->first_line.u.reply.statuscode;
 			if (method != METHOD_CANCEL) {
-				switch (status_code) {
+				switch (msg->first_line.u.reply.statuscode) {
 					case 405:
 						ret = SV_NOALLOW_405;
 						CHECK_HEADER("", allow);
@@ -1872,14 +1865,6 @@ static int w_sip_validate(struct sip_msg *msg, void *_flags, pv_spec_t* err_txt)
 				}
 			}
 
-			if (flags & SIP_PARSE_CONTACT && (status_code > 199 && status_code < 400)) {
-				ret = validate_contact_header(msg, method, reason);
-
-				if (ret != 0) {
-					goto failed;
-				}
-			}
-
 			break;
 
 		default:
@@ -2076,3 +2061,100 @@ static int get_glob_headers_values(struct sip_msg* msg, str* pattern,pv_spec_t*
 	}
 	return cnt==0 ? -1 : 1;
 }
+
+static int w_sip_to_json(struct sip_msg *msg, pv_spec_t* out_json)
+{
+	cJSON *ret=NULL, *aux, *aux2, *arr;
+	struct hdr_field* it;
+	char hdr_name_buf[255];
+	str json_ret = {0,0};
+	pv_value_t pv_val;
+
+	if (!msg) {
+		LM_ERR("No SIP msg, can't convert to json\n");
+		return -1;
+	}	
+
+	if(parse_headers(msg, HDR_EOH_F, 0) < 0) {
+		LM_ERR("Failed to parse all SIP msg \n");
+		return -1;
+	}
+
+	ret = cJSON_CreateObject();
+
+	/* first line */
+	aux = cJSON_CreateStr(msg->buf,msg->first_line.len); 
+	if (!aux) {
+		LM_ERR("Failed to create 1st line json \n");
+		goto error;
+	}
+
+	cJSON_AddItemToObject(ret,"first_line",aux);
+
+	/* headers */
+	aux = cJSON_CreateObject();
+	if (!aux) {
+		LM_ERR("Failed to create headers json \n");
+		goto error;
+	}
+
+	for (it=msg->headers;it;it=it->next) {
+		memcpy(hdr_name_buf,it->name.s,it->name.len);
+		hdr_name_buf[it->name.len] = 0;
+
+		arr = cJSON_GetObjectItem(aux, hdr_name_buf);
+		if (!arr) {
+			arr = cJSON_CreateArray();
+			cJSON_AddItemToObject(aux,hdr_name_buf,arr);
+		}
+
+		aux2 =  cJSON_CreateStr(it->body.s,it->body.len);
+		if (!aux2) {
+			LM_ERR("Failed to create individual header json\n");
+			goto error;
+		}
+
+		cJSON_AddItemToArray(arr,aux2);
+	}
+	cJSON_AddItemToObject(ret,"headers",aux);
+
+	/* body */
+	if (msg->body) {
+		aux = cJSON_CreateStr(msg->body->body.s,msg->body->body.len); 
+		if (!aux) {
+			LM_ERR("Failed to create body json\n");
+			goto error;
+		}
+		cJSON_AddItemToObject(ret,"body",aux);
+	}
+
+	json_ret.s = cJSON_Print(ret);
+	if (!json_ret.s) {
+		LM_ERR("Failed to print json to string obj\n");
+		goto error;
+	}
+
+	cJSON_Minify(json_ret.s);
+	json_ret.len = strlen(json_ret.s);
+
+	pv_val.flags = PV_VAL_STR;
+	pv_val.rs = json_ret;
+
+	if (pv_set_value(msg,out_json,0,&pv_val) != 0) {
+		LM_ERR("Failed to set out json pvar \n");
+		goto error;
+	} 
+
+	pkg_free(json_ret.s);
+	cJSON_Delete(ret);
+
+	return 1;
+
+error:
+	if (ret)
+		cJSON_Delete(ret);
+	if (json_ret.s)
+		pkg_free(json_ret.s);
+
+	return -1;
+}
