@@ -353,10 +353,10 @@ char* clen_builder(struct sip_msg* msg, int *clen_len, int diff)
  * returns 1 if cond is true, 0 if false */
 static inline int lump_check_opt(	struct lump *l,
 									struct sip_msg* msg,
-									struct socket_info* snd_s
+									const struct socket_info* snd_s
 									)
 {
-	struct ip_addr* ip;
+	const struct ip_addr* ip;
 	unsigned short port;
 	int proto;
 
@@ -443,15 +443,15 @@ static inline int lump_check_opt(	struct lump *l,
 
 /*! \brief computes the "unpacked" len of a lump list,
    code moved from build_req_from_req */
-int lumps_len(struct sip_msg* msg, struct lump* lumps,
-								struct socket_info* send_sock, int max_offset)
+static int lumps_len(struct sip_msg* msg, struct lump* lumps,
+								const struct socket_info* send_sock, int max_offset)
 {
 	unsigned int s_offset, new_len;
 	unsigned int last_del;
 	struct lump *t, *r;
-	str *send_address_str, *send_port_str;
-	str *rcv_address_str=NULL;
-	str *rcv_port_str=NULL;
+	const str *send_address_str, *send_port_str;
+	const str *rcv_address_str=NULL;
+	const str *rcv_port_str=NULL;
 
 #define SUBST_LUMP_LEN(subst_l) \
 		switch((subst_l)->u.subst){ \
@@ -751,16 +751,16 @@ void process_lumps(	struct sip_msg* msg,
 					char* new_buf,
 					unsigned int* new_buf_offs,
 					unsigned int* orig_offs,
-					struct socket_info* send_sock,
+					const struct socket_info* send_sock,
 					int max_offset)
 {
 	struct lump *t, *r;
 	char* orig;
 	unsigned int size, offset, s_offset;
 	unsigned int last_del;
-	str *send_address_str, *send_port_str;
-	str *rcv_address_str=NULL;
-	str *rcv_port_str=NULL;
+	const str *send_address_str, *send_port_str;
+	const str *rcv_address_str=NULL;
+	const str *rcv_port_str=NULL;
 
 #define SUBST_LUMP(subst_l) \
 	switch((subst_l)->u.subst){ \
@@ -1250,7 +1250,7 @@ skip_after:
  *    result (as len).
  */
 unsigned int prep_reassemble_body_parts( struct sip_msg* msg,
-												struct socket_info* send_sock)
+												const struct socket_info* send_sock)
 {
 	struct body_part *part;
 	struct lump* lump;
@@ -1585,7 +1585,7 @@ unsigned int prep_reassemble_body_parts( struct sip_msg* msg,
 
 void reassemble_body_parts( struct sip_msg* msg, char* new_buf,
 						unsigned int* new_offs, unsigned int* orig_offs,
-						struct socket_info* send_sock)
+						const struct socket_info* send_sock)
 {
 	struct body_part *part;
 	struct lump* lump;
@@ -1881,7 +1881,7 @@ void reassemble_body_parts( struct sip_msg* msg, char* new_buf,
  *   lump-based changes and body_part-based changes.
  */
 static inline int calculate_body_diff(struct sip_msg *msg,
-													struct socket_info *sock )
+													const struct socket_info *sock )
 {
 	if (msg->body==NULL) {
 		/* no body parsed, no advanced ops done, just dummy lumps over body */
@@ -1899,7 +1899,7 @@ static inline int calculate_body_diff(struct sip_msg *msg,
  */
 static inline void apply_msg_changes(struct sip_msg *msg,
 							char *new_buf, unsigned int *new_offs,
-							unsigned int *orig_offs, struct socket_info *sock,
+							unsigned int *orig_offs, const struct socket_info *sock,
 							unsigned int max_offset)
 {
 	unsigned int size;
@@ -2103,7 +2103,7 @@ int is_del_via1_lump(struct sip_msg* msg)
 
 char * build_req_buf_from_sip_req( struct sip_msg* msg,
 								unsigned int *returned_len,
-								struct socket_info* send_sock, int proto,
+								const struct socket_info* send_sock, int proto,
 								str *via_params, unsigned int flags)
 {
 	unsigned int len, new_len, received_len, rport_len, uri_len, via_len, body_delta;
@@ -2362,7 +2362,7 @@ error:
 
 
 char * build_res_buf_from_sip_res( struct sip_msg* msg,
-	unsigned int *returned_len, struct socket_info *sock,int flags)
+	unsigned int *returned_len, const struct socket_info *sock,int flags)
 {
 	unsigned int new_len, body_delta, len;
 	char *new_buf, *buf;
@@ -2779,14 +2779,14 @@ int branch_builder( unsigned int hash_index,
 
 
 char* via_builder( unsigned int *len,
-	struct socket_info* send_sock,
+	const struct socket_info* send_sock,
 	str* branch, str* extra_params, int proto, struct hostport* hp)
 {
 	unsigned int via_len, extra_len;
 	char *line_buf;
 	int max_len, local_via_len=MY_VIA_LEN;
-	str* address_str; /* address displayed in via */
-	str* port_str; /* port no displayed in via */
+	const str* address_str; /* address displayed in via */
+	const str* port_str; /* port no displayed in via */
 
 	/* use pre-set address in via or the outbound socket one */
 	if (hp && hp->host && hp->host->len)
@@ -2921,12 +2921,12 @@ char *construct_uri(str *protocol,str *username,str *domain,str *port,
 }
 
 /* uses uri_buff above, since contact is still an uri */
-char *contact_builder(struct socket_info* send_sock, int *ct_len)
+char *contact_builder(const struct socket_info* send_sock, int *ct_len)
 {
 	char *p;
 	int proto_len = 0;
-	str* address_str = get_adv_host(send_sock);
-	str* port_str = get_adv_port(send_sock);
+	const str* address_str = get_adv_host(send_sock);
+	const str* port_str = get_adv_port(send_sock);
 
 	/* sip: */
 	p = uri_buff;
