@@ -46,6 +46,8 @@
 
 #define reco_CASE     \
 	p += 4;           \
+	if (!HAVE(8))     \
+		goto other;   \
 	val = READ(p);    \
 	RD_R_CASE;        \
 	goto other;
