@@ -156,6 +156,11 @@ int receive_msg(char* buf, unsigned int len, struct receive_info* rcv_info,
 		tmp=ip_addr2a(&(rcv_info->src_ip));
 		LM_ERR("Unable to parse msg received from [%s:%d]\n",
 			tmp, rcv_info->src_port);
+		msg->buf=in_buff.s;
+		msg->len=len;
+		msg->rcv=*rcv_info;
+		msg->id=msg_no;
+		msg->msg_flags=msg_flags;
 		/* if a REQUEST msg was detected (first line was successfully parsed)
 		   we should trigger the error route */
 		if ( msg->first_line.type==SIP_REQUEST && sroutes->error.a!=NULL ) {
